/* プロパティ
 *
 * ボタン押しで終了 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include "util.h"

/* ウィンドウのプロパティのリストを出力 */

static void _put_property(Display *disp,Window win)
{
	Atom *prop;
	char **names;
	int i,num;

	prop = XListProperties(disp, win, &num);
	if(prop)
	{
		names = (char **)malloc(sizeof(char *) * num);
	
		XGetAtomNames(disp, prop, num, names);
		
		for(i = 0; i < num; i++)
		{
			if(names[i])
			{
				printf("%s\n", names[i]);
				XFree(names[i]);
			}
		}

		printf("\n");

		free(names);
		
		XFree(prop);
	}
}

/* プロパティ読み込み */

static void _read_prop(Display *disp,Window win,Atom atom)
{
	Atom type;
	int format;
	unsigned long nitem,after;
	unsigned char *buf;

	if(XGetWindowProperty(disp, win, atom, 0, 1, False, atom,
		&type, &format, &nitem, &after, &buf) == Success)
	{
		printf("read property: %ld\n", *((long *)buf));
	
		XFree(buf);
	}
}					

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	Atom atom;
	long ldat;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	printf("MaxRequestSize: %ld\n", XMaxRequestSize(disp));
	printf("ExtendedMaxRequestSize: %ld\n\n", XExtendedMaxRequestSize(disp));

	//ウィンドウ作成

	win = create_test_window(disp, ButtonPressMask | StructureNotifyMask);

	//プロパティをセット

	atom = XInternAtom(disp, "_TEST_ATOM_", False);

	ldat = 123;
	XChangeProperty(disp, win, atom, atom, 32, PropModeReplace, (unsigned char *)&ldat, 1);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		switch(ev.type)
		{
			case MapNotify:
				_put_property(disp, win);
				_read_prop(disp, win, atom);
				break;
			case ButtonPress:
				goto END;
		}
	}

END:
	XCloseDisplay(disp);

	return 0;
}
