/* ウィンドウ移動時の基準位置
 * 
 * 左ボタンで、WM_NORMAL_HINTS の win_gravity を変更して位置移動。
 * それ以外のボタンで終了。 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "util.h"

int g_gravity[3] = {NorthWestGravity, SouthEastGravity, StaticGravity};
const char *g_names[3] = {"NorthWest", "SouthEast", "Static"};

static void _set_wm_hints(Display *disp,Window win,int gravity)
{
	XSizeHints sh;

	sh.flags = PWinGravity;
	sh.win_gravity = gravity;

	XSetWMNormalHints(disp, win, &sh);
}

static void _get_winpos(Display *disp,Window win)
{
	int x,y;
	Window child;

	if(XTranslateCoordinates(disp, win, DefaultRootWindow(disp),
		0, 0, &x, &y, &child))
	{
		printf("* position: x(%d) y(%d)\n", x, y);
	}
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	int cur = -1;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	win = create_test_window(disp,
		ButtonPressMask | StructureNotifyMask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		switch(ev.type)
		{
			case ButtonPress:
				if(ev.xbutton.button != Button1) goto END;

				cur++;
				if(cur > 2) cur = 0;

				printf("* gravity: %s\n", g_names[cur]);

				_set_wm_hints(disp, win, g_gravity[cur]);

				XMoveWindow(disp, win, 0, 0);

				XMoveWindow(disp, win, 300, 300);

				_get_winpos(disp, win);
				break;
			case ConfigureNotify:
				printf("[Configure] send_event(%d) x(%d) y(%d)\n",
					ev.xconfigure.send_event, ev.xconfigure.x, ev.xconfigure.y);
				break;
		}
	}

END:
	XCloseDisplay(disp);

	return 0;
}
