/* ポップアップウィンドウ
 *
 * ウィンドウでボタンを押すと、ポップアップウィンドウ表示。
 * ポップアップ上以外でボタンを押すと、ポップアップを閉じる。
 *
 * 閉じるボタン、または右ボタンで終了。 */

#include <stdio.h>
#include <X11/Xlib.h>
#include "util.h"

#define POPUP_WIDTH  150
#define POPUP_HEIGHT 200

static Window _create_popup(Display *disp,int x,int y)
{
	XSetWindowAttributes attr;
	Window win;

	attr.background_pixel = rgb_to_pixel(0x0000ff);
	attr.override_redirect = True;

	//上位のウィンドウに渡さないイベント
	attr.do_not_propagate_mask = ButtonPressMask | ButtonReleaseMask
		| PointerMotionMask | ButtonMotionMask;

	attr.event_mask = ButtonPressMask | ButtonReleaseMask;
	
	win = XCreateWindow(disp, g_disp.root,
		x, y, POPUP_WIDTH, POPUP_HEIGHT, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWDontPropagate | CWOverrideRedirect | CWEventMask,
		&attr);

	//

	XMapWindow(disp, win);

	//ポインタグラブ

	XGrabPointer(disp, win, True,
		ButtonPressMask | ButtonReleaseMask,
		GrabModeAsync, GrabModeAsync, None, None, CurrentTime);

	return win;
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win,winpop = None;
	XEvent ev;
	int x,y;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	win = create_test_window2(disp, 300, 300, 0, ButtonPressMask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev))
		{
			if(ev.xany.window == win)
				break;
		}

		switch(ev.type)
		{
			case ButtonPress:
				x = ev.xbutton.x;
				y = ev.xbutton.y;
				
				printf("[ButtonPress] window(0x%lx) x(%d) y(%d)\n",
					ev.xbutton.window, x, y);
				
				if(ev.xbutton.button == Button3) goto END;
				
				if(ev.xbutton.window == winpop)
				{
					//ポップアップウィンドウ (グラブ中)

					if(x < 0 || y < 0 || x >= POPUP_WIDTH || y >= POPUP_HEIGHT)
					{
						//領域外なら終了 (グラブは自動的に解除)
						XDestroyWindow(disp, winpop);
						winpop = None;
						printf("* popup - close\n");
					}
				}
				else
				{
					//メインウィンドウ (グラブ中も来る)

					if(winpop)
					{
						XDestroyWindow(disp, winpop);
						winpop = None;
						printf("* main - close\n");
					}
					else
					{
						winpop = _create_popup(disp,
							ev.xbutton.x_root, ev.xbutton.y_root);
					}
				}
				break;
		}
	}

END:
	XCloseDisplay(disp);

	return 0;
}
