/* NetWM: _NET_SUPPORTED の表示 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

/* format=32 のプロパティを読み込み */

static void *_read_prop32(Display *disp,Atom prop,Atom prop_type,int *num_ret)
{
	Atom type;
	int format;
	unsigned long nitem,after;
	unsigned char *buf = NULL;

	if(XGetWindowProperty(disp, DefaultRootWindow(disp),
		prop, 0, 1024, False, prop_type,
		&type, &format, &nitem, &after, &buf) != Success)
		return NULL;

	if(!buf) return NULL;

	if(format != 32)
	{
		XFree(buf);
		return NULL;
	}

	*num_ret = nitem;

	return buf;
}

/* アトムの文字列を表示 */

static void _put_atoms(Display *disp,Atom *buf,int num)
{
	int i;
	char **names;

	names = (char **)malloc(sizeof(char *) * num);

	XGetAtomNames(disp, buf, num, names);
	
	for(i = 0; i < num; i++)
	{
		if(names[i])
		{
			printf("%s\n", names[i]);
			free(names[i]);
		}
	}

	free(names);
}

/* ウィンドウIDを表示 */

static void _put_window(Window *buf,int num)
{
	int i;

	for(i = 0; i < num; i++)
		printf("0x%lx, ", buf[i]);

	printf("\n");
}

int main(int argc,char **argv)
{
	Display *disp;
	void *buf;
	int num;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	//_NET_SUPPORTED

	buf = _read_prop32(disp, XInternAtom(disp, "_NET_SUPPORTED", False), XA_ATOM, &num);
	if(buf)
	{
		printf("[_NET_SUPPORTED]\n");
		_put_atoms(disp, (Atom *)buf, num);
		XFree(buf);
	}

	//_NET_CLIENT_LIST

	buf = _read_prop32(disp, XInternAtom(disp, "_NET_CLIENT_LIST", False), XA_WINDOW, &num);
	if(buf)
	{
		printf("\n[_NET_CLIENT_LIST]\n");
		_put_window((Window *)buf, num);
		XFree(buf);
	}

	//_NET_CLIENT_LIST_STACKING

	buf = _read_prop32(disp, XInternAtom(disp, "_NET_CLIENT_LIST_STACKING", False), XA_WINDOW, &num);
	if(buf)
	{
		printf("\n[_NET_CLIENT_LIST_STACKING]\n");
		_put_window((Window *)buf, num);
		XFree(buf);
	}

	XCloseDisplay(disp);

	return 0;
}
