/* クライアントウィンドウのプロパティ
 * 
 * 閉じるボタンで終了。 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include "util.h"

/* UTF8_STRING プロパティセット */

static void _setprop_utf8(Display *disp,Window win,Atom prop,const char *str)
{
	XChangeProperty(disp, win, prop,
		GET_ATOM("UTF8_STRING"), 8,
		PropModeReplace, (unsigned char *)str, strlen(str));
}

/* アイコンイメージセット */

static void _set_icon(Display *disp,Window win)
{
	unsigned long *buf,*pd;
	int i;
	Atom prop;

	prop = GET_ATOM("_NET_WM_ICON");

	buf = (unsigned long *)malloc((2 + 32 * 32) * sizeof(long));

	//16x16 (赤)

	buf[0] = buf[1] = 16;

	pd = buf + 2;
	for(i = 0; i < 16 * 16; i++)
		*(pd++) = 0xffff0000;

	XChangeProperty(disp, win, prop, XA_CARDINAL, 32,
		PropModeReplace, (unsigned char *)buf, 2 + 16 * 16);

	//32x32 を追加 (青、半透明)

	buf[0] = buf[1] = 32;

	pd = buf + 2;
	for(i = 0; i < 32 * 32; i++)
		*(pd++) = 0x800000ff;

	XChangeProperty(disp, win, prop, XA_CARDINAL, 32,
		PropModeAppend, (unsigned char *)buf, 2 + 32 * 32);

	free(buf);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	long *buf;
	int num;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	win = create_test_window2(disp, 200, 200, 0,
		ButtonPressMask | StructureNotifyMask);

	_setprop_utf8(disp, win, GET_ATOM("_NET_WM_NAME"), "タイトル");
	_setprop_utf8(disp, win, GET_ATOM("_NET_WM_ICON_NAME"), "ICONタイトル");
	_setprop_utf8(disp, win, GET_ATOM("_NET_WM_VISIBLE_NAME"), "タイトル[1]");
	_setprop_utf8(disp, win, GET_ATOM("_NET_WM_VISIBLE_ICON_NAME"), "ICONタイトル[1]");

	_set_icon(disp, win);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case MapNotify:
				buf = (long *)read_prop32(win, GET_ATOM("_NET_FRAME_EXTENTS"), XA_CARDINAL, &num);
				if(buf)
				{
					printf("[_NET_FRAME_EXTENTS] left:%ld, right:%ld, top:%ld, bottom:%ld\n",
						buf[0], buf[1], buf[2], buf[3]);
					
					XFree(buf);
				}
				break;
		}
	}

	XCloseDisplay(disp);

	return 0;
}
