/* StartupNotify
 *
 * 閉じるボタンで終了 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include "util.h"

/* 送る文字列の作成
 *
 * ※<"><\> が含まれていないものとして扱う。
 * ret_len: ヌル文字も含む長さ */

static char *_get_notify_string(char *notify_id,int *ret_len)
{
	char *buf;
	int len;

	len = strlen(notify_id);

	buf = (char *)malloc(len + 14);

	memcpy(buf, "remove: ID=\"", 12);
	memcpy(buf + 12, notify_id, len);
	buf[12 + len] = '"';
	buf[13 + len] = 0;

	*ret_len = len + 14;

	return buf;
}

/* 完了通知を送る */

static void _send_notify(Display *disp,char *notify_id)
{
	Window win;
	XClientMessageEvent ev;
	int len,pos,size;
	char *str;

	str = _get_notify_string(notify_id, &len);

	printf("[send] %s\n", str);

	win = XCreateWindow(disp, g_disp.root,
		 0, 0, 1, 1, 0,
		 CopyFromParent, InputOnly, CopyFromParent,
		 0, NULL);

	//イベント

	memset(&ev, 0, sizeof(XClientMessageEvent));

	ev.type = ClientMessage;
	ev.message_type = GET_ATOM("_NET_STARTUP_INFO_BEGIN");
	ev.display = disp;
	ev.window = win;
	ev.format = 8;

	//送信

	pos = 0;

	while(len > 0)
	{
		size = (len < 20)? len: 20;
		
		memcpy(ev.data.b, str + pos, size);

		if(size < 20)
			memset(ev.data.b + size, 0, 20 - size);

		XSendEvent(disp, g_disp.root, False, PropertyChangeMask, (XEvent *)&ev);

		if(pos == 0)
			ev.message_type = GET_ATOM("_NET_STARTUP_INFO");

		pos += size;
		len -= size;
	}

	//終了

	XDestroyWindow(disp, win);
	XFlush(disp);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win[2],leader;
	XEvent ev;
	XWMHints h;
	char *notify_id;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//環境変数の取得

	notify_id = getenv("DESKTOP_STARTUP_ID");

	if(!notify_id)
		printf("DESKTOP_STARTUP_ID: <none>\n");
	else
	{
		notify_id = strdup(notify_id);

		printf("DESKTOP_STARTUP_ID: '%s'\n", notify_id);
	}

	unsetenv("DESKTOP_STARTUP_ID");

	//グループリーダー

	leader = XCreateWindow(disp, g_disp.root,
		0, 0, 1, 1, 0,
		CopyFromParent, InputOnly, CopyFromParent,
		0, NULL);

	if(notify_id)
	{
		XChangeProperty(disp, leader,
			GET_ATOM("_NET_STARTUP_ID"), GET_ATOM("UTF8_STRING"), 8,
			PropModeReplace, (unsigned char *)notify_id, strlen(notify_id));
	}

	//通常ウィンドウ

	win[0] = create_test_window2(disp, 200, 200, 0, StructureNotifyMask);

	win[1] = create_test_window2(disp, 200, 200,
		rgb_to_pixel(0x0000ff), StructureNotifyMask);

	//グループリーダーをセット

	h.flags = WindowGroupHint;
	h.window_group = leader;

	XSetWMHints(disp, win[0], &h);
	XSetWMHints(disp, win[1], &h);

	//イベント

	XMapWindow(disp, win[0]);
	XMapWindow(disp, win[1]);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case MapNotify:
				//最初のマップ時に通知
				if(notify_id)
				{
					_send_notify(disp, notify_id);
					
					free(notify_id);
					notify_id = NULL;
				}
				break;
		}
	}

	XCloseDisplay(disp);

	if(notify_id) free(notify_id);

	return 0;
}
