/* _NET_WM_USER_TIME
 *
 * 左クリックでウィンドウを作成。
 * 閉じるボタンで、閉じる or 終了 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include "util.h"

static Window _create_window(int width,int height,unsigned int bkgnd,Window leader)
{
	Window win;
	XWMHints h;

	win = create_test_window2(g_disp.disp,
		width, height, rgb_to_pixel(bkgnd),
		ButtonPressMask | KeyPressMask);

	XChangeProperty(g_disp.disp, win,
		GET_ATOM("_NET_WM_USER_TIME_WINDOW"),
		XA_WINDOW, 32, PropModeReplace, (unsigned char *)&leader, 1);

	h.flags = WindowGroupHint;
	h.window_group = leader;

	XSetWMHints(g_disp.disp, win, &h);

	return win;
}

static void _set_user_time(Window win,unsigned long time)
{
	XChangeProperty(g_disp.disp, win,
		GET_ATOM("_NET_WM_USER_TIME"),
		XA_CARDINAL, 32, PropModeReplace, (unsigned char *)&time, 1);

	printf("[usertime] 0x%lx\n", time);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win,leader,popup = None;
	XEvent ev;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//グループリーダー

	leader = XCreateWindow(disp, g_disp.root,
		0, 0, 1, 1, 0,
		CopyFromParent, InputOnly, CopyFromParent,
		0, NULL);

	//通常ウィンドウ

	win = _create_window(200, 200, 0, leader);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev))
		{
			if(ev.xany.window == popup)
			{
				XDestroyWindow(disp, popup);
				popup = None;
				continue;
			}
			else
				break;
		}

		switch(ev.type)
		{
			case ButtonPress:
				_set_user_time(leader, ev.xbutton.time);

				if(ev.xbutton.button == Button1 && !popup)
				{
					popup = _create_window(150, 150, 0x0000ff, leader);

					XMapWindow(disp, popup);
				}
				break;
			case KeyPress:
				_set_user_time(leader, ev.xkey.time);
				break;
		}
	}

	XCloseDisplay(disp);

	return 0;
}
