/* _NET_WM_STATE
 *
 * 初期状態で最大化にする。
 * 右ボタンで最大化の切り替え。
 * 右ボタンで全画面化の切り替え。
 * 閉じるボタンで終了。 */

#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include "util.h"

enum
{
	_ACTION_ADD = 0,
	_ACTION_DELETE,
	_ACTION_TOGGLE
};

/* プロパティのアトムリストを出力 */

static void _put_atom_list(Window win,Atom atom,const char *name,int state)
{
	Atom *buf;
	int num;

	if(state == PropertyDelete)
	{
		printf("[%s] <delete>\n", name);
		return;
	}

	buf = read_prop32(win, atom, XA_ATOM, &num);
	if(!buf) return;

	printf("[%s] (%d) ", name, num);

	put_atoms_name((Atom *)buf, num);

	XFree(buf);
}

/* ウィンドウの状態を変更 */

static void _change_state(Window win,Atom prop,int action,Atom state1,Atom state2)
{
	XClientMessageEvent ev;

	memset(&ev, 0, sizeof(XClientMessageEvent));

	ev.type = ClientMessage;
	ev.message_type = prop;
	ev.window = win;
	ev.format = 32;
	ev.data.l[0] = action;
	ev.data.l[1] = state1;
	ev.data.l[2] = state2;
	ev.data.l[3] = 1;

	XSendEvent(g_disp.disp, g_disp.root, False,
		SubstructureNotifyMask | SubstructureRedirectMask,
		(XEvent *)&ev);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	Atom atoms[5];
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	atoms[0] = GET_ATOM("_NET_WM_STATE");
	atoms[1] = GET_ATOM("_NET_WM_ALLOWED_ACTIONS");
	atoms[2] = GET_ATOM("_NET_WM_STATE_FULLSCREEN");
	atoms[3] = GET_ATOM("_NET_WM_STATE_MAXIMIZED_HORZ");
	atoms[4] = GET_ATOM("_NET_WM_STATE_MAXIMIZED_VERT");

	win = create_test_window2(disp, 200, 200, 0,
		ButtonPressMask | PropertyChangeMask | StructureNotifyMask);

	//初期状態で最大化

	XChangeProperty(disp, win, atoms[0],
		XA_ATOM, 32, PropModeReplace, (unsigned char *)&atoms[3], 2);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case ButtonPress:
				if(ev.xbutton.button == Button1)
				{
					printf("* toggle MAXIMIZE\n");
					_change_state(win, atoms[0], _ACTION_TOGGLE, atoms[3], atoms[4]);
				}
				else if(ev.xbutton.button == Button3)
				{
					printf("* toggle FULLSCREEN\n");
					_change_state(win, atoms[0], _ACTION_TOGGLE, atoms[2], None);
				}
				break;
			case MapNotify:
				printf("* Map\n");
				break;
			case PropertyNotify:
				if(ev.xproperty.atom == atoms[0])
				{
					_put_atom_list(win, atoms[0], "_NET_WM_STATE", ev.xproperty.state);
				}
				else if(ev.xproperty.atom == atoms[1])
				{
					_put_atom_list(win, atoms[1], "_NET_WM_ALLOWED_ACTIONS",
						ev.xproperty.state);
				}
				break;
		}
	}

	XCloseDisplay(disp);

	return 0;
}
