/* _NET_WM_MOVERESIZE
 *
 * 左ボタンドラッグで、ウィンドウ位置の移動。
 * 右ボタンドラッグで、ウィンドウのリサイズ (右下)。
 *
 * 閉じるボタンで終了。 */

#include <string.h>
#include <X11/Xlib.h>
#include "util.h"

#define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
#define _NET_WM_MOVERESIZE_SIZE_TOP          1
#define _NET_WM_MOVERESIZE_SIZE_TOPRIGHT     2
#define _NET_WM_MOVERESIZE_SIZE_RIGHT        3
#define _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT  4
#define _NET_WM_MOVERESIZE_SIZE_BOTTOM       5
#define _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT   6
#define _NET_WM_MOVERESIZE_SIZE_LEFT         7
#define _NET_WM_MOVERESIZE_MOVE              8  //移動
#define _NET_WM_MOVERESIZE_SIZE_KEYBOARD     9  //キーボードによるサイズ変更
#define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10  //キーボードによる位置移動
#define _NET_WM_MOVERESIZE_CANCEL           11  //キャンセル

/* 位置移動・リサイズを開始 */

static void _moveresize(Window win,int direction,int x_root,int y_root,int button)
{
	XClientMessageEvent ev;

	//自動グラブを解除
	XUngrabPointer(g_disp.disp, CurrentTime);

	memset(&ev, 0, sizeof(XClientMessageEvent));

	ev.type = ClientMessage;
	ev.message_type = GET_ATOM("_NET_WM_MOVERESIZE");
	ev.window = win;
	ev.format = 32;
	ev.data.l[0] = x_root;
	ev.data.l[1] = y_root;
	ev.data.l[2] = direction;
	ev.data.l[3] = button;
	ev.data.l[4] = 1;

	XSendEvent(g_disp.disp, g_disp.root, False,
		SubstructureNotifyMask | SubstructureRedirectMask,
		(XEvent *)&ev);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	win = create_test_window2(disp, 200, 200, 0, ButtonPressMask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case ButtonPress:
				if(ev.xbutton.button == Button1)
				{
					_moveresize(win, _NET_WM_MOVERESIZE_MOVE,
						ev.xbutton.x_root, ev.xbutton.y_root, ev.xbutton.button);
				}
				else if(ev.xbutton.button == Button3)
				{
					_moveresize(win, _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT,
						ev.xbutton.x_root, ev.xbutton.y_root, ev.xbutton.button);
				}
				break;
		}
	}

	XCloseDisplay(disp);

	return 0;
}
