/* クリップボード1
 *
 * 左クリックで、CLIPBOARD セレクションの TARGETS,TIMESTAMP を表示。
 * その他のボタンで、CLIPBOARD セレクションから UTF8_STRING を取得して表示。
 *
 * 閉じるボタンで終了 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include "util.h"

enum
{
	_ATOM_CLIPBOARD,
	_ATOM_MULTIPLE,
	_ATOM_ATOM_PAIR,
	_ATOM_MY_SELECTION,
	_ATOM_TARGETS,
	_ATOM_TIMESTAMP,
	_ATOM_UTF8_STRING,

	_ATOM_NUM
};

const char *g_names[] = {
	"CLIPBOARD", "MULTIPLE", "ATOM_PAIR", "_MY_SELECTION_",
	"TARGETS", "TIMESTAMP", "UTF8_STRING"
};

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	Atom atoms[_ATOM_NUM],multi[4];
	char *str;
	int size;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	XInternAtoms(disp, (char **)g_names, _ATOM_NUM, False, atoms);

	win = create_test_window2(disp, 200, 200, 0, ButtonPress);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case ButtonPress:
				if(ev.xbutton.button == Button1)
				{
					//MULTIPLE 用のアトムペア
					multi[0] = multi[1] = atoms[_ATOM_TARGETS];
					multi[2] = multi[3] = atoms[_ATOM_TIMESTAMP];

					XChangeProperty(disp, win, atoms[_ATOM_MY_SELECTION],
						atoms[_ATOM_ATOM_PAIR], 32, PropModeReplace,
						(unsigned char *)multi, 4);

					//MULTIPLE を要求
					XConvertSelection(disp, atoms[_ATOM_CLIPBOARD],
						atoms[_ATOM_MULTIPLE], atoms[_ATOM_MY_SELECTION],
						win, ev.xbutton.time);
				}
				else
				{
					//UTF8_STRING を要求
					XConvertSelection(disp, atoms[_ATOM_CLIPBOARD],
						atoms[_ATOM_UTF8_STRING], atoms[_ATOM_MY_SELECTION],
						win, ev.xbutton.time);
				}

				printf("* XConvertSelection: time(0x%lx)\n", ev.xbutton.time);
				break;

			case SelectionNotify:
				printf("[SelectionNotify] requestor(0x%lx) property(%lu) time(0x%lx)\n",
					ev.xselection.requestor, ev.xselection.property,
					ev.xselection.time);

				if(ev.xselection.property == None) break;

				if(ev.xselection.target == atoms[_ATOM_MULTIPLE])
				{
					//MULTIPLE

					put_prop_atoms(win, atoms[_ATOM_TARGETS], "TARGETS");
					put_prop_num(win, atoms[_ATOM_TIMESTAMP], XA_INTEGER, "TIMESTAMP");

					XDeleteProperty(disp, win, atoms[_ATOM_TARGETS]);
					XDeleteProperty(disp, win, atoms[_ATOM_TIMESTAMP]);
					XDeleteProperty(disp, win, atoms[_ATOM_MY_SELECTION]);
				}
				else if(ev.xselection.target == atoms[_ATOM_UTF8_STRING])
				{
					//UTF8_STRING

					str = read_prop8(win, atoms[_ATOM_MY_SELECTION], NULL, &size);
					if(str)
					{
						printf("--- UTF8_STRING (size:%d) ---\n%s\n-----------\n", size, str);
						free(str);
					}

					XDeleteProperty(disp, win, atoms[_ATOM_MY_SELECTION]);
				}
				break;
		}
	}

	XCloseDisplay(disp);

	return 0;
}
