/* クリップボード3
 *
 * 左クリックで、所有権を取得。
 * 他のクライアントにテキストを貼り付け可能。
 *
 * 閉じるボタンで終了 */

#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include "util.h"

enum
{
	_ATOM_CLIPBOARD,
	_ATOM_MY_SELECTION,
	_ATOM_ATOM_PAIR,
	_ATOM_TARGETS,
	_ATOM_TIMESTAMP,
	_ATOM_MULTIPLE,
	_ATOM_UTF8_STRING,

	_ATOM_NUM
};

const char *g_names[] = {
	"CLIPBOARD", "_MY_SELECTION_", "ATOM_PAIR",
	"TARGETS", "TIMESTAMP", "MULTIPLE", "UTF8_STRING"
};

Atom atoms[_ATOM_NUM];
Time g_timestamp = 0;

const char *copy_string = "テストABCDEFG_";

/* 各ターゲットをセット */

static int _set_property(Window requestor,Atom property,Atom target)
{
	if(target == atoms[_ATOM_TARGETS])
	{
		//TARGETS
		//(TARGETS, TIMESTAMP, MULTIPLE, UTF8_STRING)
		
		XChangeProperty(g_disp.disp, requestor, property,
			XA_ATOM, 32,
			PropModeReplace, (unsigned char *)&atoms[_ATOM_TARGETS], 4);

		return 0;
	}
	else if(target == atoms[_ATOM_TIMESTAMP])
	{
		//TIMESTAMP
		
		XChangeProperty(g_disp.disp, requestor, property,
			XA_INTEGER, 32,
			PropModeReplace, (unsigned char *)&g_timestamp, 1);

		return 0;
	}
	else if(target == atoms[_ATOM_UTF8_STRING])
	{
		//UTF8_STRING
		
		XChangeProperty(g_disp.disp, requestor, property,
			atoms[_ATOM_UTF8_STRING], 8,
			PropModeReplace, (unsigned char *)copy_string, strlen(copy_string));

		return 0;
	}

	return 1;
}

/* MULTIPLE */

static int _set_multiple(Window requestor,Atom property)
{
	Atom *buf,*ps;
	int i,num;

	buf = (Atom *)read_prop32(requestor, property,
		atoms[_ATOM_ATOM_PAIR], &num);
	if(!buf) return 1;

	ps = buf;

	for(i = 0; i < num / 2; i++, ps += 2)
		_set_property(requestor, ps[1], ps[0]);

	XFree(buf);

	return 0;
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	XSelectionEvent evs;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	XInternAtoms(disp, (char **)g_names, _ATOM_NUM, False, atoms);

	win = create_test_window2(disp, 200, 200, 0, ButtonPress);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case ButtonPress:
				if(ev.xbutton.button == Button1)
				{
					g_timestamp = ev.xbutton.time;
					
					XSetSelectionOwner(disp, atoms[_ATOM_CLIPBOARD], win, g_timestamp);
				}
				break;
			case SelectionClear:
				printf("[SelectionClear] time(0x%lx)\n", ev.xselectionclear.time);
				break;
			case SelectionRequest:
				printf("[SelectionRequest] requestor(0x%lx) time(0x%lx) target(",
					ev.xselectionrequest.requestor, ev.xselectionrequest.time);

				put_atom_name(ev.xselectionrequest.target);
				printf(")\n");

				//所有者がプロパティを選択
				if(ev.xselectionrequest.property == None)
					ev.xselectionrequest.property = ev.xselectionrequest.target;

				if(ev.xselectionrequest.target == atoms[_ATOM_MULTIPLE])
				{
					//MULTIPLE

					if(_set_multiple(ev.xselectionrequest.requestor,
						ev.xselectionrequest.property))
						ev.xselectionrequest.property = None;
				}
				else
				{
					if(_set_property(ev.xselectionrequest.requestor,
						ev.xselectionrequest.property,
						ev.xselectionrequest.target))
					{
						ev.xselectionrequest.property = None;
					}
				}

				//SelectionNotify を送る

				evs.type = SelectionNotify;
				evs.requestor = ev.xselectionrequest.requestor;
				evs.selection = ev.xselectionrequest.selection;
				evs.target = ev.xselectionrequest.target;
				evs.property = ev.xselectionrequest.property;
				evs.time = ev.xselectionrequest.time;

				XSendEvent(disp, evs.requestor, False, 0, (XEvent *)&evs);
				break;
		}
	}

	XCloseDisplay(disp);

	return 0;
}
