/* D&D を受け取る。
 *
 * 青い部分にドロップすると受け入れる。
 * 黒い部分ではドロップを拒否する。
 *
 * 閉じるボタンで終了 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include "util.h"

enum
{
	_ATOM_XdndSelection,
	_ATOM_XdndAware,
	_ATOM_XdndTypeList,
	_ATOM_XdndEnter,
	_ATOM_XdndPosition,
	_ATOM_XdndStatus,
	_ATOM_XdndLeave,
	_ATOM_XdndDrop,
	_ATOM_XdndFinished,
	_ATOM_URI_LIST,
	_ATOM_MY_SELECTION,

	_ATOM_NUM
};

const char *g_names[] = {
	"XdndSelection", "XdndAware", "XdndTypeList",
	"XdndEnter", "XdndPosition", "XdndStatus", "XdndLeave",
	"XdndDrop", "XdndFinished", "text/uri-list",
	"_MY_SELECTION_"
};

Atom atoms[_ATOM_NUM], g_action = None;
Window g_dnd_src = None;
int g_width,g_height;

/* ClientMessage 送信 */

static void _send_message(int type,unsigned long val1,unsigned long val2,
	unsigned long val3,unsigned long val4,unsigned long val5)
{
	XClientMessageEvent ev;

	ev.type = ClientMessage;
	ev.window = g_dnd_src;
	ev.message_type = atoms[type];
	ev.format = 32;
	ev.data.l[0] = val1;
	ev.data.l[1] = val2;
	ev.data.l[2] = val3;
	ev.data.l[3] = val4;
	ev.data.l[4] = val5;

	XSendEvent(g_disp.disp, g_dnd_src, False, 0, (XEvent *)&ev);
}

/* XdndEnter */

static void _dnd_enter(XClientMessageEvent *ev)
{
	int i;

	g_dnd_src = (Window)ev->data.l[0];

	printf("[XdndEnter] src(0x%lx) flags(0x%lx:ver=%d)",
		ev->data.l[0], ev->data.l[1],
		(unsigned int)ev->data.l[1] >> 24);

	for(i = 2; i <= 4; i++)
	{
		printf(" type%d(", i - 1);
		put_atom_name(ev->data.l[i]);
		printf(")");
	}

	printf("\n");

	//4つ以上のタイプ

	if(ev->data.l[1] & 1)
		put_prop_atoms(g_dnd_src, atoms[_ATOM_XdndTypeList], "XdndTypeList");

	//ソースウィンドウの破棄を通知するようにする

	XSelectInput(g_disp.disp, g_dnd_src, StructureNotifyMask);
}

/* XdndPosition */

static void _dnd_position(XClientMessageEvent *ev)
{
	int x,y,flag;
	Window child;

	if(ev->data.l[0] != g_dnd_src) return;

	x = (unsigned long)ev->data.l[2] >> 16;
	y = ev->data.l[2] & 0xffff;

	printf("[XdndPosition] src(0x%lx) x(%d) y(%d) time(0x%lx) action(",
		ev->data.l[0], x, y, ev->data.l[3]);

	put_atom_name(ev->data.l[4]);
	printf(")\n");

	g_action = ev->data.l[4];

	//XdndStatus

	XTranslateCoordinates(g_disp.disp, g_disp.root, ev->window,
		x, y, &x, &y, &child);

	flag = (x < g_width / 2);

	_send_message(_ATOM_XdndStatus, ev->window,
		(flag)? 3: 0, 0, 0, g_action);
}

/* XdndDrop */

static void _dnd_drop(XClientMessageEvent *ev)
{
	if(ev->data.l[0] != g_dnd_src) return;

	printf("[XdndDrop] src(0x%lx) time(0x%lx)\n",
		ev->data.l[0], ev->data.l[2]);

	XConvertSelection(g_disp.disp, atoms[_ATOM_XdndSelection],
		atoms[_ATOM_URI_LIST], atoms[_ATOM_MY_SELECTION],
		ev->window, (Time)ev->data.l[2]);
}

static void _client_message(XClientMessageEvent *ev)
{
	Atom mestype;

	mestype = ev->message_type;

	if(mestype == atoms[_ATOM_XdndEnter])
		_dnd_enter(ev);
	else if(mestype == atoms[_ATOM_XdndPosition])
		_dnd_position(ev);
	else if(mestype == atoms[_ATOM_XdndLeave])
	{
		if(g_dnd_src == (Window)ev->data.l[0])
		{
			printf("[XdndLeave]\n");
			g_dnd_src = None;
		}
	}
	else if(mestype == atoms[_ATOM_XdndDrop])
		_dnd_drop(ev);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	char *buf;
	int size;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	XInternAtoms(disp, (char **)g_names, _ATOM_NUM, False, atoms);

	win = create_test_window2(disp, 200, 200, 0,
		StructureNotifyMask | ExposureMask);

	g_width = 200;
	g_height = 200;

	set_prop32_one(win, atoms[_ATOM_XdndAware], XA_ATOM, 5);

	XSetForeground(disp, g_disp.gc, rgb_to_pixel(0x0000ff));

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case Expose:
				XFillRectangle(disp, win, g_disp.gc,
					0, 0, g_width / 2, g_height);
				break;
			case ClientMessage:
				_client_message((XClientMessageEvent *)&ev);
				break;
			case ConfigureNotify:
				g_width = ev.xconfigure.width;
				g_height = ev.xconfigure.height;
				break;
			case SelectionNotify:
				printf("[SelectionNotify] target(");
				put_atom_name(ev.xselection.target);
				printf(")\n");

				if(ev.xselection.property != None)
				{
					buf = (char *)read_prop8(ev.xselection.requestor,
						ev.xselection.property, NULL, &size);

					if(buf)
					{
						printf("-----------\n");
						fwrite(buf, 1, size, stdout);
						printf("\n-----------\n");

						free(buf);
					}

					XDeleteProperty(disp, ev.xselection.requestor,
						ev.xselection.property);
				}

				//データを受け取ったら、D&D終了
				_send_message(_ATOM_XdndFinished, win, 1, g_action, 0, 0);
				g_dnd_src = None;
				break;
			case DestroyNotify:
				if(g_dnd_src && ev.xdestroywindow.window == g_dnd_src)
				{
					printf("* Destroy src\n");
					g_dnd_src = None;
				}
				break;
		}
	}

	XCloseDisplay(disp);

	return 0;
}
