/* DoubleBuffer 拡張機能
 *
 * -lXext
 *
 * 左ボタン押しで、フロント/バックバッファを入れ替え。
 * 閉じるボタンで終了。 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/extensions/Xdbe.h>
#include "util.h"

static void _put_info(Display *disp)
{
	XdbeScreenVisualInfo *info;
	XdbeVisualInfo *vi;
	int i,num = 1;

	info = XdbeGetVisualInfo(disp, &g_disp.root, &num);
	if(!info) return;

	vi = info->visinfo;

	for(i = 0; i < info->count; i++)
	{
		printf("[%d] visual(0x%lx) depth(%d) plevel(%d)\n",
			i, vi[i].visual, vi[i].depth, vi[i].perflevel);
	}

	XdbeFreeVisualInfo(info);
}

static void _draw(Display *disp,Drawable dst,int count)
{
	int rgb;

	rgb = (count)? 0xff0000: 0x00ff00;

	XSetForeground(disp, g_disp.gc, rgb_to_pixel(rgb));

	XFillRectangle(disp, dst, g_disp.gc, 0, 0, 200, 200);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win,back;
	XEvent ev;
	XdbeSwapInfo swap;
	int major,minor,count = 0;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//DBE

	if(!XdbeQueryExtension(disp, &major, &minor))
		printf("unsupported DBE\n");
	else
		printf("DBE ver %d.%d\n", major, minor);

	_put_info(disp);

	//

	win = create_test_window2(disp, 200, 200, 0, ButtonPress);

	//バックバッファ

	back = XdbeAllocateBackBufferName(disp, win, XdbeUndefined);

	_draw(disp, back, count);

	swap.swap_window = win;
	swap.swap_action = XdbeUntouched;

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		if(ev.type == ButtonPress && ev.xbutton.button == Button1)
		{
			XdbeSwapBuffers(disp, &swap, 1);

			count ^= 1;
			_draw(disp, back, count);
		}
	}

	XCloseDisplay(disp);

	return 0;
}
