/* Render 拡張機能、情報取得
 *
 * -lXrender */

#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/extensions/Xrender.h>

static void _put_bits(XRenderDirectFormat *p)
{
	char m[34];
	int i,n;

	memset(m, '0', 32);

	for(i = 31 - p->red, n = 1; n < p->redMask + 1; i--, n <<= 1)
		m[i] = 'R';

	for(i = 31 - p->green, n = 1; n < p->greenMask + 1; i--, n <<= 1)
		m[i] = 'G';

	for(i = 31 - p->blue, n = 1; n < p->blueMask + 1; i--, n <<= 1)
		m[i] = 'B';

	for(i = 31 - p->alpha, n = 1; n < p->alphaMask + 1; i--, n <<= 1)
		m[i] = 'A';

	m[32] = 0;
	printf("%s\n", m);
}

static void _put_pictformat(XRenderPictFormat *pf)
{
	printf("id(0x%lx) type(%s) depth(%d)\n",
		pf->id, (pf->type == PictTypeIndexed)? "Indexed": "Direct",
		pf->depth);

	if(pf->type == PictTypeDirect)
	{
		printf("r(%d) g(%d) b(%d) a(%d)"
			" rmask(0x%x) gmask(0x%x) bmask(0x%x) amask(0x%x)\n",
			pf->direct.red, pf->direct.green, pf->direct.blue, pf->direct.alpha,
			pf->direct.redMask, pf->direct.greenMask, pf->direct.blueMask,
			pf->direct.alphaMask);

		_put_bits(&pf->direct);
	}

	printf("\n");
}

int main(int argc,char **argv)
{
	Display *disp;
	XRenderPictFormat *pf,temp;
	int major,minor,i;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	//バージョン

	if(!XRenderQueryVersion(disp, &major, &minor))
		printf("unsupported XRender\n");
	else
		printf("XRender ver %d.%d\n", major, minor);

	//standard

	for(i = 0; i <= 4; i++)
	{
		printf("-- standard %d --\n", i);
	
		pf = XRenderFindStandardFormat(disp, i);
		if(pf) _put_pictformat(pf);
	}

	//default visual

	printf("-- default visual --\n");

	i = DefaultScreen(disp);

	pf = XRenderFindVisualFormat(disp, DefaultVisual(disp, i));
	if(pf) _put_pictformat(pf);

	//Direct

	temp.type = PictTypeDirect;

	for(i = 0; 1; i++)
	{
		pf = XRenderFindFormat(disp, PictFormatType, &temp, i);
		if(!pf) break;

		printf("-- Direct %d --\n", i);
		_put_pictformat(pf);
	}

	XCloseDisplay(disp);

	return 0;
}
