/* Render 拡張機能、Pitcure
 *
 * -lXext -lXrender
 * 
 * 青のウィンドウに、赤のアルファグラデーションを合成する。
 * 閉じるボタンで終了する。
 */

#include <stdint.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#include <X11/extensions/Xrender.h>
#include "util.h"

XShmSegmentInfo g_seginfo;

#define WIDTH  200
#define HEIGHT 200

static Pixmap _create_image(Display *disp)
{
	Pixmap pixmap;
	uint32_t *pd,a;
	int ix,iy;

	//depth が 32 以下なら、Y1列のバイト数を BitmapPad(display) のビット単位にすること

	g_seginfo.shmid = shmget(IPC_PRIVATE, WIDTH * 4 * HEIGHT, IPC_CREAT | 0777);
	g_seginfo.shmaddr = shmat(g_seginfo.shmid, 0, 0);
	g_seginfo.readOnly = False;

	XShmAttach(disp, &g_seginfo);

	pixmap = XShmCreatePixmap(disp, g_disp.root, g_seginfo.shmaddr,
		&g_seginfo, WIDTH, HEIGHT, 32);

	//赤 + A グラデーション

	pd = (uint32_t *)g_seginfo.shmaddr;

	for(iy = 0; iy < HEIGHT; iy++)
	{
		for(ix = 0; ix < WIDTH; ix++)
		{
			a = ix * 255 / (WIDTH - 1);
			*(pd++) = 0xff0000 | (a << 24);
		}
	}

	return pixmap;
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	Pixmap pixmap;
	Picture picwin,picimg;
	XRenderPictFormat *pfwin,*pfimg;
	int major,minor;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	XShmQueryExtension(disp);

	//Render

	if(!XRenderQueryVersion(disp, &major, &minor))
		printf("unsupported XRender\n");
	else
		printf("XRender ver %d.%d\n", major, minor);

	//

	win = create_test_window2(disp, WIDTH, HEIGHT, rgb_to_pixel(0x0000ff), ExposureMask);

	//ウィンドウの Picture

	pfwin = XRenderFindVisualFormat(disp, DefaultVisualOfScreen(g_disp.screen));
	picwin = XRenderCreatePicture(disp, win, pfwin, 0, NULL);

	//ソース

	pixmap = _create_image(disp);

	pfimg = XRenderFindStandardFormat(disp, PictStandardARGB32);
	picimg = XRenderCreatePicture(disp, pixmap, pfimg, 0, NULL);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case Expose:
				XRenderComposite(disp, PictOpOver, picimg, picimg, picwin,
					0,0, 0,0, 0,0, WIDTH, HEIGHT);
				break;
		}
	}

	//

	XShmDetach(disp, &g_seginfo);
	shmdt(g_seginfo.shmaddr);
	shmctl(g_seginfo.shmid, IPC_RMID, 0);

	XFreePixmap(disp, pixmap);

	XCloseDisplay(disp);

	return 0;
}
