/* カーソル
 *
 * 左ボタンを押すごとに、カーソルを変更する。
 * ２色の十字カーソル → 青のグラデーションカーソル → デフォルトテーマの "wait" カーソル
 *
 * 閉じるボタンで終了。 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xcursor/Xcursor.h>
#include "util.h"

const unsigned char g_curimg_1bit[] = {
15,15, //width,height
7,7,   //xhot, yhot
//image
0xc0,0x01,0x40,0x01,0x40,0x01,0x40,0x01,0x40,0x01,0x40,0x01,0x3f,0x7e,0x01,0x40,
0x3f,0x7e,0x40,0x01,0x40,0x01,0x40,0x01,0x40,0x01,0x40,0x01,0xc0,0x01,
//mask
0xc0,0x01,0xc0,0x01,0xc0,0x01,0xc0,0x01,0xc0,0x01,0xc0,0x01,0xff,0x7f,0xff,0x7f,
0xff,0x7f,0xc0,0x01,0xc0,0x01,0xc0,0x01,0xc0,0x01,0xc0,0x01,0xc0,0x01 };

/* 2色カーソル */

static Cursor _create_cursor2(Display *disp,const unsigned char *buf)
{
	Cursor cur;
	Pixmap img,mask;
	XColor colf,colb;
	int w,h;

	w = buf[0];
	h = buf[1];

	//Pixmap 作成

	img = XCreateBitmapFromData(disp, g_disp.root, (char *)buf + 4, w, h);

	mask = XCreateBitmapFromData(disp, g_disp.root,
			(char *)buf + 4 + ((w + 7) / 8) * h, w, h);

	//Cursor 作成

	colf.red = colf.green = colf.blue = 0;
	colf.flags = DoRed | DoGreen | DoBlue;

	colb.red = 0xffff;
	colb.green = colb.blue = 0;
	colb.flags = DoRed | DoGreen | DoBlue;

	cur = XCreatePixmapCursor(disp, img, mask, &colf, &colb, buf[2], buf[3]);

	XFreePixmap(disp, img);
	XFreePixmap(disp, mask);

	return cur;
}

/* ARGB カーソル */

static Cursor _create_cursorARGB(Display *disp)
{
	XcursorImage *img;
	XcursorPixel *pd;
	Cursor cur;
	int ix,iy,a;

	img = XcursorImageCreate(16, 8);
	if(!img) return 0;

	img->xhot = 0;
	img->yhot = 0;

	//イメージ

	pd = img->pixels;

	for(iy = 0; iy < 8; iy++)
	{
		for(ix = 0; ix < 16; ix++)
		{
			a = 255 - ix * 255 / 15;
			*(pd++) = ((uint32_t)a << 24) | 0x0000ff;
		}
	}

	//カーソルに変換

	cur = XcursorImageLoadCursor(disp, img);

	XcursorImageDestroy(img);

	return cur;
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	Cursor cur[3];
	int current = 0;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	printf("[path] %s\n", XcursorLibraryPath());

	cur[0] = _create_cursor2(disp, g_curimg_1bit);
	cur[1] = _create_cursorARGB(disp);
	cur[2] = XcursorLibraryLoadCursor(disp, "wait");

	win = create_test_window2(disp, 200, 200, rgb_to_pixel(0xffffff), ButtonPress);

	XDefineCursor(disp, win, cur[0]);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		if(ev.type == ButtonPress && ev.xbutton.button == Button1)
		{
			current++;
			if(current >= 3) current = 0;

			XDefineCursor(disp, win, cur[current]);
		}
	}

	XCloseDisplay(disp);

	return 0;
}
