/* XInput2 : デバイス情報(クラス)
 *
 * -lXi */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/extensions/XInput2.h>
#include "util.h"

//keyclass

static void _put_keyclass(XIKeyClassInfo *p)
{
	int i,*pcode;

	printf("<KeyClass> [srcid:%d] num:%d\n ", p->sourceid, p->num_keycodes);

	pcode = p->keycodes;

	for(i = 0; i < p->num_keycodes; i++)
		printf("%d,", *(pcode++));

	printf("\n");
}

//buttonclass

static void _put_buttonclass(XIButtonClassInfo *p)
{
	int i;

	printf("<ButtonClass> [srcid:%d] num:%d\n", p->sourceid, p->num_buttons);

	//labels

	for(i = 0; i < p->num_buttons; i++)
	{
		printf("btn%d: ", i + 1);

		if(p->labels[i])
		{
			put_atom_name(p->labels[i]);
			printf("\n");
		}
		else
			printf("<None>\n");
	}

	//state

	printf("state:(len:%d) ", p->state.mask_len);

	for(i = 0; i < p->state.mask_len; i++)
		printf("0x%02X ", p->state.mask[i]);

	printf("\n");
}

//valuator class

static void _put_valclass(XIValuatorClassInfo *p)
{
	printf("<ValuatorClass> [srcid:%d]\n"
	" number(%d) label(",
		p->sourceid, p->number);

	put_atom_name(p->label);
	printf(")\n");

	printf(" min(%.3f) max(%.3f) value(%.3f) reso(%d) mode(%s)\n",
		p->min, p->max, p->value, p->resolution,
		(p->mode == XIModeAbsolute)? "Abs": "Rel");
}

//scroll class

static void _put_scrollclass(XIScrollClassInfo *p)
{
	printf("<ScrollClass> [srcid:%d]\n"
	" number(%d) type(%s) inc(%.3f) flags(0x%X)\n",
		p->sourceid, p->number,
		(p->scroll_type == XIScrollTypeVertical)? "Vert": "Horz",
		p->increment, p->flags);
}

int main(int argc,char **argv)
{
	Display *disp;
	XIDeviceInfo *info,*pi;
	XIAnyClassInfo **ppclass,*pclass;
	int major,minor,dnum,i,j,cnum;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//version

	major = 2, minor = 4;

	if(XIQueryVersion(disp, &major, &minor) != Success)
	{
		printf("unsupported XI2\n");
		XCloseDisplay(disp);
		return 1;
	}

	//デバイス情報

	info = XIQueryDevice(disp, XIAllDevices, &dnum);

	for(i = 0; i < dnum; i++)
	{
		pi = info + i;

		printf("\n---- [%s] (%d) ---\n", pi->name, pi->deviceid);

		cnum = pi->num_classes;
		ppclass = pi->classes;

		for(j = 0; j < cnum; j++, ppclass++)
		{
			pclass = *ppclass;

			switch(pclass->type)
			{
				case XIKeyClass:
					_put_keyclass((XIKeyClassInfo *)pclass);
					break;
				case XIButtonClass:
					_put_buttonclass((XIButtonClassInfo *)pclass);
					break;
				case XIValuatorClass:
					_put_valclass((XIValuatorClassInfo *)pclass);
					break;
				case XIScrollClass:
					_put_scrollclass((XIScrollClassInfo *)pclass);
					break;
			}
		}
	}

	XIFreeDeviceInfo(info);

	//

	XCloseDisplay(disp);

	return 0;
}
