/* XInput: XI_DeviceChanged イベント
 *
 * -lXi
 * 
 * 閉じるボタンで終了する。 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/extensions/XInput2.h>
#include "util.h"

static void _xi2_event(XIEvent *ev)
{
	XIDeviceChangedEvent *p;

	if(ev->evtype == XI_DeviceChanged)
	{
		p = (XIDeviceChangedEvent *)ev;
		
		printf("<XI_DeviceChanged>\n"
		" deviceid(%d) sourceid(%d) reason(%d) num_classes(%d)\n",
			p->deviceid, p->sourceid, p->reason, p->num_classes);
	}
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	void *data;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	if(xi2_init())
	{
		printf("unsupported XI2\n");
		XCloseDisplay(disp);
		return 1;
	}

	//

	win = create_test_window2(disp, 200, 200, 0,
		ButtonPressMask | KeyPressMask);

	xi2_select_event(win, XIAllMasterDevices, XI_DeviceChangedMask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case ButtonPress:
				printf("* ButtonPress\n");
				break;
			case KeyPress:
				printf("* KeyPress\n");
				break;
			case GenericEvent:
				data = xi2_get_event(&ev);
				if(data)
				{
					_xi2_event((XIEvent *)data);
					
					XFreeEventData(disp, (XGenericEventCookie *)&ev);
				}
				break;
		}
	}

	//

	XCloseDisplay(disp);

	return 0;
}
