/* XInput: ポインタとキーのイベント
 *
 * -lXi
 * 
 * 閉じるボタンで終了する。 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/extensions/XInput2.h>
#include "util.h"

static unsigned int g_count = 0;

static void _xi2_event(XIDeviceEvent *ev)
{
	int i;
	double *dval;

	printf("[%u] ", g_count++);

	switch(ev->evtype)
	{
		case XI_KeyPress:
			printf("<XI_KeyPress>\n");
			break;
		case XI_KeyRelease:
			printf("<XI_KeyRelease>\n");
			break;
		case XI_ButtonPress:
			printf("<XI_ButtonPress>\n");
			break;
		case XI_ButtonRelease:
			printf("<XI_ButtonRelease>\n");
			break;
		case XI_Motion:
			printf("<XI_Motion>\n");
			break;
	}

	printf(" deviceid(%d) sourceid(%d) detail(%d)"
		" root(0x%04x) event(0x%04x) child(0x%04x)\n"
		" root_x(%.3f) root_y(%.3f) event_x(%.3f) event_y(%.3f) flags(0x%x)\n",
		ev->deviceid, ev->sourceid, ev->detail,
		(unsigned int)ev->root, (unsigned int)ev->event, (unsigned int)ev->child,
		ev->root_x, ev->root_y, ev->event_x, ev->event_y, ev->flags);

	//buttons

	printf(" buttons(len:%d): ", ev->buttons.mask_len);

	for(i = 0; i < ev->buttons.mask_len * 8; i++)
	{
		if(ev->buttons.mask[i / 8] & (1 << (i & 7)))
			printf("btn%d,", i);
	}

	printf("\n");

	//valuators

	printf(" valuators(len:%d): ", ev->valuators.mask_len);

	dval = ev->valuators.values;

	for(i = 0; i < ev->valuators.mask_len * 8; i++)
	{
		if(ev->valuators.mask[i / 8] & (1 << (i & 7)))
			printf("val%d(%.3f),", i, *(dval++));
	}

	printf("\n");

	//mods

	printf(" mods: base(0x%x) latched(0x%x) locked(0x%x) effective(0x%x)\n",
		ev->mods.base, ev->mods.latched, ev->mods.locked, ev->mods.effective);

	//group

	printf(" group: base(%d) latched(%d) locked(%d) effective(%d)\n",
		ev->group.base, ev->group.latched, ev->group.locked, ev->group.effective);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XEvent ev;
	void *data;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	if(xi2_init())
	{
		printf("unsupported XI2\n");
		XCloseDisplay(disp);
		return 1;
	}

	//

	win = create_test_window2(disp, 200, 200, 0,
		ButtonPressMask | KeyPressMask);

	xi2_select_event(win,
		(argc < 2)? XIAllMasterDevices: atoi(argv[1]),
		XI_KeyPressMask | XI_KeyReleaseMask
			| XI_ButtonPressMask | XI_ButtonReleaseMask | XI_MotionMask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(event_quit(&ev)) break;

		switch(ev.type)
		{
			case ButtonPress:
				printf("* ButtonPress\n");
				break;
			case KeyPress:
				printf("* KeyPress\n");
				break;
			case GenericEvent:
				data = xi2_get_event(&ev);
				if(data)
				{
					_xi2_event((XIDeviceEvent *)data);
					
					XFreeEventData(disp, (XGenericEventCookie *)&ev);
				}
				break;
		}
	}

	//

	XCloseDisplay(disp);

	return 0;
}
