/* XKB 拡張機能
 *
 * 閉じるボタンで終了。 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include "util.h"

static unsigned int g_count = 0;

static void _put_state(XkbStateNotifyEvent *ev)
{
	unsigned int f = ev->changed;
	
	printf("----[%d]-----\n"
		"device: %d / changed: 0x%x\n",
		g_count++, ev->device, ev->changed);

	if(f & XkbModifierStateMask) printf("mods: 0x%x\n", ev->mods);
	if(f & XkbModifierBaseMask) printf("base_mods: 0x%x\n", ev->base_mods);
	if(f & XkbModifierLatchMask) printf("latched_mods: 0x%x\n", ev->latched_mods);
	if(f & XkbModifierLockMask) printf("locked_mods: 0x%x\n", ev->locked_mods);

	if(f & XkbGroupStateMask) printf("group: %d\n", ev->group);
	if(f & XkbGroupBaseMask) printf("base_group: %d\n", ev->base_group);
	if(f & XkbGroupLatchMask) printf("latched_group: %d\n", ev->latched_group);
	if(f & XkbGroupLockMask) printf("locked_group: %d\n", ev->locked_group);

	if(f & XkbCompatStateMask) printf("compat_state: 0x%x\n", ev->compat_state);

	if(f & XkbGrabModsMask) printf("grab_mods: 0x%x\n", ev->grab_mods);
	if(f & XkbCompatGrabModsMask) printf("compat_grab_mods: 0x%x\n", ev->compat_grab_mods);

	if(f & XkbLookupModsMask) printf("lookup_mods: 0x%x\n", ev->lookup_mods);
	if(f & XkbCompatLookupModsMask) printf("compat_lookup_mods: 0x%x\n", ev->compat_lookup_mods);

	//if(f & XkbPointerButtonMask) printf("ptr_buttons: 0x%x\n", ev->ptr_buttons);
	
	printf("keycode: %d / event_type: %d / req_major: %d / req_minor: %d\n",
		ev->keycode, ev->event_type, ev->req_major, ev->req_minor);
}

int main(int argc,char **argv)
{
	Display *disp;
	Window win;
	XkbEvent ev;
	int major,minor,opcode,event_type,error_base;
	unsigned long mask;
	
	//XKB ライブラリ

	major = XkbMajorVersion;
	minor = XkbMinorVersion;

	if(!XkbLibraryVersion(&major, &minor))
	{
		printf("unsupported XKB library\n");
		return 1;
	}

	printf("library: ver %d.%d\n", major, minor);

	//

	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//XKB 初期化

	if(!XkbQueryExtension(disp, &opcode, &event_type, &error_base,
		&major, &minor))
	{
		XCloseDisplay(disp);
		printf("unsupported XKB (server)\n");
		return 1;
	}

	printf("server: ver %d.%d\n", major, minor);
	printf("opcode: %d\nevent:%d\nerror:%d\n\n", opcode, event_type, error_base);

	//

	win = create_test_window2(disp, 200, 200, 0, ButtonPress);

	//XKB イベント選択 (ポインタボタン以外)

	mask = XkbAllStateComponentsMask - XkbPointerButtonMask;

	XkbSelectEventDetails(disp, XkbUseCoreKbd, XkbStateNotify, mask, mask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, (XEvent *)&ev);

		if(event_quit((XEvent *)&ev)) break;

		if(ev.type == event_type
			&& ev.any.xkb_type == XkbStateNotify)
		{
			_put_state((XkbStateNotifyEvent *)&ev);
		}
	}

	XCloseDisplay(disp);

	return 0;
}
