/* XKB: クライアントマップ */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include "util.h"

static void _put_types(XkbClientMapPtr p)
{
	int i,j;
	XkbKeyTypePtr pt;
	XkbKTMapEntryPtr pm;
	XkbModsPtr ppr;

	printf("size_types: %d\n"
		"num_types: %d\n",
		p->size_types, p->num_types);

	pt = p->types;

	for(i = 0; i < p->num_types; i++, pt++)
	{
		printf("\n--- [%d] ---\n", i);
		put_atom_name_line("name: ", pt->name);
		
		printf("mods: mask(0x%x) real(0x%x) vmods(0x%x)\n"
		"num_levels: %d\n",
		pt->mods.mask, pt->mods.real_mods, pt->mods.vmods, pt->num_levels);

		for(j = 0; j < pt->num_levels; j++)
		{
			printf("level[%d]: ", j);
			put_atom_name_line(NULL, pt->level_names[j]);
		}

		//map

		pm = pt->map;

		for(j = 0; j < pt->map_count; j++, pm++)
		{
			printf("<map[%d]> active(%d) level(%d) mods[mask(0x%x):real(0x%x):vmods(0x%x)]\n",
				j, pm->active, pm->level,
				pm->mods.mask, pm->mods.real_mods, pm->mods.vmods);
		}

		//preserve

		if(pt->preserve)
		{
			ppr = pt->preserve;
			
			for(j = 0; j < pt->map_count; j++, ppr++)
			{
				if(ppr->mask)
				{
					printf("<preserve[%d]> mask(0x%x):real(0x%x):vmods(0x%x)\n",
						j, ppr->mask, ppr->real_mods, ppr->vmods);
				}
			}
		}
	}
}

static void _put_symmap(XkbDescPtr p,int min,int max)
{
	int i,j,group_num;
	XkbSymMapPtr pm;
	KeySym *sym;
	char *name;

	printf("\n==== key_sym_map ====\n\n");

	pm = p->map->key_sym_map + min;

	for(i = min; i <= max; i++, pm++)
	{
		printf("[%d]<%.4s> kt_index(%d,%d,%d,%d)"
		" group_info(0x%x) width(%d) offset(%d)\n",
			i, p->names->keys[i].name,
			pm->kt_index[0], pm->kt_index[1],
			pm->kt_index[2], pm->kt_index[3],
			pm->group_info, pm->width, pm->offset);

		sym = p->map->syms + pm->offset;
		group_num = pm->group_info & 0x0f;

		for(j = group_num * pm->width; j; j--)
		{
			name = XKeysymToString(*(sym++));
			if(name)
				printf(" %s", name);
			else
				printf(" <null>");
		}

		printf("\n");
	}
}

int main(int argc,char **argv)
{
	Display *disp;
	XkbDescPtr desc;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//XKB 初期化

	if(init_xkb(disp, NULL))
		return 1;

	//データ取得

	desc = XkbGetKeyboard(disp, XkbAllComponentsMask, XkbUseCoreKbd);

	_put_types(desc->map);

	_put_symmap(desc, desc->min_key_code, desc->max_key_code);

	XkbFreeKeyboard(desc, 0, True);

	XCloseDisplay(disp);

	return 0;
}
