/* ユーティリティ関数 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/XKBlib.h>
#include "util.h"

//---------------

dispinfo g_disp;

static const char *g_atom_names[] = {
	"WM_PROTOCOLS",
	"WM_DELETE_WINDOW"
};

//---------------


/* 最初にビットが 1 になる位置 */

static int _get_biton(unsigned long val)
{
	int i;

	for(i = 0; i < 32; i++)
	{
		if(val & (1 << i)) return i;
	}

	return -1;
}

/* ビットが連続で 1 になっている数 */

static int _get_bitnum(unsigned long val)
{
	int i;

	for(i = 0; i < 8; i++)
	{
		if(!(val & (1 << i))) break;
	}

	return i;
}

/* X エラーハンドラ */

static int _xerr_handle(Display *display,XErrorEvent *ev)
{
	char m[256],no[16];

	fprintf(stderr,
		"--- X error ---\n"
		" type(%d) serial(%lu) error_code(%d)\n"
		" request_code(%d) minor_code(%d) resourceid(0x%lx)\n",
		ev->type, ev->serial, ev->error_code,
		ev->request_code, ev->minor_code, ev->resourceid);

	XGetErrorText(display, ev->error_code, m, 256);
	fprintf(stderr, " [Error] %s\n", m);

	snprintf(no, 16, "%d", ev->request_code);
	XGetErrorDatabaseText(display, "XRequest", no, "default error", m, 256);
	fprintf(stderr, " [Request] %s\n", m);

	fprintf(stderr, "------------\n");
	fflush(stderr);

	return 1;
}

//------------------

/* ディスプレイ情報セット */

void set_display(Display *d)
{
	dispinfo *p = &g_disp;
	Visual *v;

	XSetErrorHandler(_xerr_handle);

	p->disp = d;
	p->def_screen_no = DefaultScreen(d);
	p->screen = DefaultScreenOfDisplay(d);

	p->root = RootWindowOfScreen(p->screen);
	p->depth = DefaultDepthOfScreen(p->screen);

	p->gc = XCreateGC(d, p->root, 0, NULL);

	XInternAtoms(d, (char **)g_atom_names, 2, False, p->atoms);

	//ビジュアル

	v = DefaultVisualOfScreen(p->screen);

	p->r_shift = _get_biton(v->red_mask);
	p->g_shift = _get_biton(v->green_mask);
	p->b_shift = _get_biton(v->blue_mask);

	p->r_bits = _get_bitnum(v->red_mask >> p->r_shift);
	p->g_bits = _get_bitnum(v->green_mask >> p->g_shift);
	p->b_bits = _get_bitnum(v->blue_mask >> p->b_shift);

	p->r_max = (1 << p->r_bits) - 1;
	p->g_max = (1 << p->g_bits) - 1;
	p->b_max = (1 << p->b_bits) - 1;
}

/* ロケールのセット */

void init_locale(void)
{
	setlocale(LC_ALL, "");

	if(XSupportsLocale())
		XSetLocaleModifiers("");
}

/* ウィンドウの作成 */

Window create_test_window(Display *disp,long event_mask)
{
	XSetWindowAttributes attr;
	Window win;

	attr.background_pixel = 0;
	attr.event_mask = event_mask;

	win = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, 200, 200, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWEventMask, &attr);

	return win;
}

/* ウィンドウの作成 (WM_PROTOCOLS あり)
 *
 * bkgnd: 負の値で背景色なし */

Window create_test_window2(Display *disp,int width,int height,int bkgnd,long event_mask)
{
	XSetWindowAttributes attr;
	Window win;
	int mask;

	mask = CWEventMask;
	attr.event_mask = event_mask;

	if(bkgnd >= 0)
	{
		mask |= CWBackPixel;
		attr.background_pixel = bkgnd;
	}

	win = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, width, height, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		 mask, &attr);

	set_window_protocols(win);

	return win;
}

/* WM_PROTOCOLS プロパティをセット */

void set_window_protocols(Window win)
{
	XSetWMProtocols(g_disp.disp, win, g_disp.atoms + 1, 1);
}

/* イベントループ (ボタン押しで終了) */

void eventloop_button_quit(Display *disp)
{
	XEvent ev;

	while(1)
	{
		XNextEvent(disp, &ev);

		if(ev.type == ButtonPress)
			break;
	}
}

/* WM_DELETE_WINDOW のイベントかどうか */

int event_quit(XEvent *ev)
{
	if(ev->type == ClientMessage
		&& ev->xclient.message_type == g_disp.atoms[0]
		&& (Atom)ev->xclient.data.l[0] == g_disp.atoms[1])
	{
		//WM_DELETE_WINDOW
		return 1;
	}

	return 0;
}

/* RGB値からピクセル値を取得 */

unsigned int rgb_to_pixel(unsigned int col)
{
	int r,g,b;

	r = (col >> 16) & 255;
	g = (col >> 8) & 255;
	b = col & 255;

	if(g_disp.depth <= 16)
	{
		r = r * g_disp.r_max / 255;
		g = g * g_disp.g_max / 255;
		b = b * g_disp.b_max / 255;
	}

	return (r << g_disp.r_shift) | (g << g_disp.g_shift) | (b << g_disp.b_shift);
}

/* R,G,B 値からピクセル値を取得 */

unsigned int rgb_to_pixel_sep(int r,int g,int b)
{
	if(g_disp.depth <= 16)
	{
		r = r * g_disp.r_max / 255;
		g = g * g_disp.g_max / 255;
		b = b * g_disp.b_max / 255;
	}

	return (r << g_disp.r_shift) | (g << g_disp.g_shift) | (b << g_disp.b_shift);
}

/* キー・ボタンの state の状態を表示 */

void put_state(unsigned int s)
{
	printf("  state(0x%x", s);

	if(s & Button1Mask) printf(":Btn1");
	if(s & Button2Mask) printf(":Btn2");
	if(s & Button3Mask) printf(":Btn3");
	if(s & Button4Mask) printf(":Btn4");
	if(s & Button5Mask) printf(":Btn5");
	if(s & ShiftMask) printf(":Shift");
	if(s & ControlMask) printf(":Ctrl");
	if(s & Mod1Mask) printf(":Mod1");
	if(s & Mod2Mask) printf(":Mod2");
	if(s & Mod3Mask) printf(":Mod3");
	if(s & Mod4Mask) printf(":Mod4");
	if(s & Mod5Mask) printf(":Mod5");
	if(s & LockMask) printf(":Lock");

	printf(")\n");
}

/* XImage バッファにピクセル値をセット */

void ximage_setbuf(XImage *img,unsigned char *buf,unsigned int pixcol)
{
	switch(img->bits_per_pixel)
	{
		case 32:
			*((uint32_t *)buf) = pixcol;
			break;
		case 24:
			if(img->byte_order == LSBFirst)
			{
				buf[0] = (uint8_t)pixcol;
				buf[1] = (uint8_t)(pixcol >> 8);
				buf[2] = (uint8_t)(pixcol >> 16);
			}
			else
			{
				buf[0] = (uint8_t)(pixcol >> 16);
				buf[1] = (uint8_t)(pixcol >> 8);
				buf[2] = (uint8_t)pixcol;
			}
			break;
		case 16:
			*((uint16_t *)buf) = pixcol;
			break;
	}
}

//=======================
// 入力メソッド
//=======================

/* Nothing or None の入力スタイルの組み合わせを取得
 *
 * return: 0 でなし */

XIMStyle get_xim_style_nothing(XIM im)
{
	XIMStyles *styles;
	XIMStyle ret = 0,s;
	int i;

	XGetIMValues(im, XNQueryInputStyle, &styles, (void *)0);

	for(i = 0; i < styles->count_styles; i++)
	{
		s = styles->supported_styles[i];

		if((s & XIMPreeditNothing) && (s & XIMStatusNothing))
			ret = s;
		else if((s & XIMPreeditNothing) && (s & XIMStatusNone))
			ret = s;
		else if((s & XIMPreeditNone) && (s & XIMStatusNothing))
			ret = s;
		else if((s & XIMPreeditNone) && (s & XIMStatusNone))
			ret = s;

		if(ret) break;
	}

	XFree(styles);

	return ret;
}

/* 指定した前編集スタイルと、XIMStatusNothing or XIMStatusNone の入力スタイルの組み合わせを取得
 *
 * return: 0 でなし */

XIMStyle get_xim_style(XIM im,XIMStyle preedit)
{
	XIMStyles *styles;
	XIMStyle ret = 0,s;
	int i;

	XGetIMValues(im, XNQueryInputStyle, &styles, (void *)0);

	for(i = 0; i < styles->count_styles; i++)
	{
		s = styles->supported_styles[i];

		if((s & preedit) && (s & XIMStatusNothing))
			ret = s;
		else if((s & preedit) && (s & XIMStatusNone))
			ret = s;

		if(ret) break;
	}

	XFree(styles);

	return ret;
}

/* KeyPress 時、入力コンテキストの文字列を出力 */

void put_xic_text(XEvent *ev,XIC ic)
{
	char m[64],*buf;
	KeySym keysym;
	Status ret;
	int len;

	buf = m;

	len = XmbLookupString(ic, (XKeyEvent *)ev, m, 63, &keysym, &ret);

	//バッファ長さが足りない

	if(ret == XBufferOverflow)
	{
		buf = (char *)malloc(len + 1);
		if(!buf) return;

		len = XmbLookupString(ic, (XKeyEvent *)ev, buf, len, &keysym, &ret);
	}

	//表示

	buf[len] = 0;

	if(ret == XLookupBoth || ret == XLookupKeySym)
		printf("# KeySym: %lu\n", keysym);

	if(ret == XLookupBoth || ret == XLookupChars)
	{
		if(len == 1 && !(buf[0] >= 0x20 && buf[0] < 0x7f))
			//ASCII 制御文字
			printf("# <%d>\n", buf[0]);
		else
			printf("# '%s'\n", buf);
	}

	if(buf != m) free(buf);
}

//=======================
// プロパティ
//=======================

/* format=32 のプロパティを読み込み
 *
 * num_ret: 数が返る
 * return: XFree() で解放する。NULL で失敗。 */

void *read_prop32(Window win,Atom prop,Atom prop_type,int *num_ret)
{
	Atom type;
	int format;
	unsigned long nitem,after;
	unsigned char *buf = NULL;

	if(XGetWindowProperty(g_disp.disp, win,
		prop, 0, 1024, False, prop_type,
		&type, &format, &nitem, &after, &buf) != Success)
		return NULL;

	if(!buf) return NULL;

	if(format != 32)
	{
		XFree(buf);
		return NULL;
	}

	*num_ret = nitem;

	return buf;
}

/* format=32 のプロパティを配列で指定個読み込み
 *
 * return: バッファにセットされた数 */

int read_prop32_array(Window win,Atom prop,Atom prop_type,void *buf,int num)
{
	void *read;
	int pnum;

	read = read_prop32(win, prop, prop_type, &pnum);
	if(!read) return 0;

	if(num > pnum) num = pnum;

	memcpy(buf, read, num * sizeof(long));

	XFree(read);

	return num;
}

/* プロパティから format=8 データ取得
 *
 * 終端には常に 0 がある。(サイズには含まない)
 * return: free() で解放する */

void *read_prop8(Window win,Atom prop,Atom *ret_type,int *ret_size)
{
	Atom type;
	int ret,format;
	unsigned long nitems,after,size;
	long offset;
	unsigned char *pdat,*buf;

	//データサイズ取得

	ret = XGetWindowProperty(g_disp.disp, win, prop, 0, 0,
			False, AnyPropertyType, &type, &format, &nitems, &after, &pdat);

	if(pdat) XFree(pdat);

	if(ret != Success || type == None || format != 8)
		return NULL;

	//確保
	
	size = after;
	
	buf = (unsigned char *)malloc(size + 1);
	if(!buf) return NULL;

	if(ret_type) *ret_type = type;
	if(ret_size) *ret_size = size;
	
	//読み込み (4byte単位)

	for(offset = 0; after > 0; offset += nitems)
	{
		ret = XGetWindowProperty(g_disp.disp, win, prop,
				offset / 4, after / 4 + 1,
				False, AnyPropertyType, &type, &format, &nitems, &after, &pdat);

		if(ret != Success)
		{
			free(buf);
			return NULL;
		}

		memcpy(buf + offset, pdat, nitems);
		
		XFree(pdat);
	}
	
	//NULL 追加
	
	buf[size] = 0;

	return (void *)buf;
}

/* format=32 のプロパティをセット */

void set_prop32_one(Window win,Atom prop,Atom prop_type,long val)
{
	XChangeProperty(g_disp.disp, win, prop, prop_type, 32,
		PropModeReplace, (unsigned char *)&val, 1);
}

/* format=32 のプロパティをセット (配列) */

void set_prop32_array(Window win,Atom prop,Atom prop_type,void *buf,int num)
{
	XChangeProperty(g_disp.disp, win, prop, prop_type, 32,
		PropModeReplace, (unsigned char *)buf, num);
}

/* アトムの名前を表示 */

void put_atom_name(Atom atom)
{
	char *name;

	if(atom == None)
		printf("0");
	else
	{
		name = XGetAtomName(g_disp.disp, atom);
		if(name)
		{
			printf("%s", name);
			XFree(name);
		}
	}
}

/* アトムの名前を1行で表示 */

void put_atom_name_line(const char *prefix,Atom atom)
{
	char *name;

	if(prefix) fputs(prefix, stdout);

	if(atom == None)
		printf("<None>\n");
	else
	{
		name = XGetAtomName(g_disp.disp, atom);
		if(name)
		{
			printf("\"%s\"\n", name);
			XFree(name);
		}
	}
}

/* アトムの配列からアトム名を表示 */

void put_atoms_name(Atom *buf,int num)
{
	int i;
	char **names;

	names = (char **)malloc(sizeof(char *) * num);

	XGetAtomNames(g_disp.disp, buf, num, names);
	
	for(i = 0; i < num; i++)
	{
		if(names[i])
		{
			printf("%s, ", names[i]);
			free(names[i]);
		}
	}

	free(names);

	printf("\n");
}

/* プロパティからアトムの配列を読み込み、名前を表示 */

void put_prop_atoms(Window win,Atom prop,const char *name)
{
	void *buf;
	int num;

	printf("<%s> ", name);

	buf = read_prop32(win, prop, XA_ATOM, &num);
	if(!buf)
	{
		printf("<none>\n");
		return;
	}

	printf("(%d) ", num);

	put_atoms_name((Atom *)buf, num);

	XFree(buf);
}

/* プロパティから数値の配列を読み込み、表示 */

void put_prop_num(Window win,Atom prop,Atom proptype,const char *name)
{
	long *buf;
	int i,num;

	printf("<%s> ", name);

	buf = (long *)read_prop32(win, prop, proptype, &num);
	if(!buf)
	{
		printf("<none>\n");
		return;
	}

	for(i = 0; i < num; i++)
		printf("%ld, ", buf[i]);

	printf("\n");

	XFree(buf);
}

//=================

/* XKB 初期化 */

int init_xkb(Display *disp,int *event_type)
{
	int major,minor,opcode,error_base,ev;

	//XKB ライブラリ

	major = XkbMajorVersion;
	minor = XkbMinorVersion;

	if(!XkbLibraryVersion(&major, &minor))
	{
		printf("unsupported XKB library\n");
		XCloseDisplay(disp);
		return 1;
	}

	//XKB 初期化

	if(!XkbQueryExtension(disp, &opcode, &ev, &error_base,
		&major, &minor))
	{
		printf("unsupported XKB (server)\n");
		XCloseDisplay(disp);
		return 1;
	}

	if(event_type) *event_type = ev;

	return 0;
}
