/* XI2 用ユーティリティ関数 */

#include <X11/Xlib.h>
#include <X11/extensions/XInput2.h>
#include "util.h"


/* XI2 初期化 */

int xi2_init(void)
{
	int major,minor,event,error;

	if(!XQueryExtension(g_disp.disp, "XInputExtension", &g_disp.opcode, &event, &error))
		return 1;

	major = 2, minor = 4;

	if(XIQueryVersion(g_disp.disp, &major, &minor) != Success)
		return 1;

	return 0;
}

/* イベント選択 */

void xi2_select_event(Window win,int deviceid,unsigned int mask)
{
	XIEventMask e;
	unsigned char dat[4]; //一応4byteとして用意

	dat[0] = mask & 0xff;
	dat[1] = (mask >> 8) & 0xff;
	dat[2] = (mask >> 16) & 0xff;
	dat[3] = (mask >> 24) & 0xff;

	e.deviceid = deviceid;
	e.mask_len = 4;
	e.mask = dat;

	XISelectEvents(g_disp.disp, win, &e, 1);
}

/* XI2 のイベント取得
 *
 * 戻り値が NULL でない場合、XFreeEventData(disp, (XGenericEventCookie *)&ev) を行うこと。
 *
 * return: NULL で XI2 のイベントではない or エラー。イベント構造体に型変換して使う。 */

void *xi2_get_event(XEvent *ev)
{
	if(ev->xcookie.extension != g_disp.opcode)
		return NULL;
	else
	{
		if(!XGetEventData(g_disp.disp, (XGenericEventCookie *)ev))
			return NULL;

		return ev->xcookie.data;
	}
}


