/* ポインタのグラブ
 *
 * 背景が黒と白の２つのウィンドウが表示されます。
 *
 * Button1 (左ボタン) 押しで、そのウィンドウでグラブ開始、離すとグラブ解放。
 * Space キー押しで、アクティブなウィンドウでグラブ開始/解放。
 * Enter キー押しで終了 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window win[2];
	XEvent ev;
	KeySym key;
	int i,fgrab = 0;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	//ウィンドウ作成

	for(i = 0; i < 2; i++)
	{
		attr.background_pixel = (i)? 0xffffff: 0;
		attr.event_mask = ButtonPressMask | ButtonReleaseMask
			| PointerMotionMask | ButtonMotionMask
			| EnterWindowMask | LeaveWindowMask | KeyPressMask;

		win[i] = XCreateWindow(disp, DefaultRootWindow(disp),
			i * 100, 0, 200, 200, 0,
			CopyFromParent, CopyFromParent, CopyFromParent,
			CWBackPixel | CWEventMask, &attr);

		XMapWindow(disp, win[i]);
	}

	//イベント

	while(1)
	{
		XNextEvent(disp, &ev);

		switch(ev.type)
		{
			case ButtonPress:
				printf("* [Press] x(%d) y(%d) button(%d)\n",
					ev.xbutton.x, ev.xbutton.y, ev.xbutton.button);

				if(fgrab == 0 && ev.xbutton.button == Button1)
				{
					XGrabPointer(disp, ev.xbutton.window, False,
						ButtonPressMask | ButtonReleaseMask | EnterWindowMask | LeaveWindowMask
						| PointerMotionMask | ButtonMotionMask,
						GrabModeAsync, GrabModeAsync, None, None, CurrentTime);

					fgrab = 1;
					printf("# Grab - Button1\n");
				}

				fflush(stdout);
				break;
			case ButtonRelease:
				if(fgrab == 1 && ev.xbutton.button == Button1)
				{
					XUngrabPointer(disp, CurrentTime);
					fgrab = 0;

					printf("# Ungrab - Button1\n");
				}
				break;
			case MotionNotify:
				printf("- [Motion] x(%d) y(%d)\n", ev.xmotion.x, ev.xmotion.y);
				fflush(stdout);
				break;
			case EnterNotify:
			case LeaveNotify:
				printf("$ [%s] <%s> x(%d) y(%d) mode(%d) detail(%d)\n",
					(ev.type == EnterNotify)? "Enter": "Leave",
					(ev.xcrossing.window == win[0])? "black": "white",
					ev.xcrossing.x, ev.xcrossing.y,
					ev.xcrossing.mode, ev.xcrossing.detail);
				fflush(stdout);
				break;

			case KeyPress:
				key = XLookupKeysym((XKeyEvent *)&ev, 0);
				if(key == XK_Return || key == XK_KP_Enter)
					goto END;
				else if(key == ' ')
				{
					if(fgrab == 2)
					{
						XUngrabPointer(disp, CurrentTime);
						fgrab = 0;
						printf("# Ungrab - key\n");
					}
					else if(fgrab == 0)
					{
						XGrabPointer(disp, ev.xkey.window, False,
							ButtonPressMask | ButtonReleaseMask | EnterWindowMask | LeaveWindowMask
							| PointerMotionMask | ButtonMotionMask,
							GrabModeAsync, GrabModeAsync, None, None, CurrentTime);

						fgrab = 2;
						printf("# Grab - key\n");
					}
				}
				break;
		}
	}

END:
	XCloseDisplay(disp);

	return 0;
}

