/* 親のパッシブグラブの続行/解除
 *
 * 緑部分がフレームウィンドウ、その子として黒の中身ウィンドウを作成。
 * 緑部分を左ドラッグすると、パッシブグラブによるグラブ (カーソル変化)。
 * 黒部分を左ドラッグすると、親のパッシブグラブを解除して、通常通りのイベントを取得。
 * 左ボタン以外を押すと終了。 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include "util.h"

#define FRAME_WIDTH 15

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window winf,winc;
	XEvent ev;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//フレームウィンドウ作成

	attr.background_pixel = rgb_to_pixel(0x00ff00);
	attr.event_mask = ButtonPressMask | ButtonReleaseMask | ButtonMotionMask;
	attr.override_redirect = True;

	winf = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, 200, 200, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWEventMask | CWOverrideRedirect, &attr);

	XGrabButton(disp, Button1, AnyModifier, winf, False,
		ButtonPressMask | ButtonReleaseMask | ButtonMotionMask,
		GrabModeSync, GrabModeAsync, None, XCreateFontCursor(disp, XC_hand1));

	//中身のウィンドウ作成

	attr.background_pixel = 0;
	attr.event_mask = ButtonPressMask | ButtonReleaseMask | ButtonMotionMask;

	winc = XCreateWindow(disp, winf,
		FRAME_WIDTH, FRAME_WIDTH, 200 - FRAME_WIDTH * 2, 200 - FRAME_WIDTH * 2, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWEventMask, &attr);

	//イベント

	XMapWindow(disp, winc);
	XMapWindow(disp, winf);

	while(1)
	{
		XNextEvent(disp, &ev);

		switch(ev.type)
		{
			case ButtonPress:
			case ButtonRelease:
				printf("* [%s] <%s> x(%d) y(%d) button(%d)\n",
					(ev.type == ButtonPress)? "Press": "Release",
					(ev.xbutton.window == winc)? "content": "frame",
					ev.xbutton.x, ev.xbutton.y, ev.xbutton.button);

				fflush(stdout);

				if(ev.type == ButtonPress)
				{
					if(ev.xbutton.button == Button1)
					{
						if(ev.xbutton.window == winf)
						{
							XAllowEvents(disp,
								(ev.xbutton.subwindow)? ReplayPointer: AsyncPointer,
								ev.xbutton.time);

							XSync(disp, 0);
						}
					}
					else
						goto END;
				}
				break;
			case MotionNotify:
				printf("- [Motion] <%s> x(%d) y(%d)\n",
					(ev.xmotion.window == winc)? "content": "frame",
					ev.xmotion.x, ev.xmotion.y);
				fflush(stdout);
				break;
		}
	}

END:
	XCloseDisplay(disp);

	return 0;
}

