/* ポインタのボタンマッピング変更
 *
 * 左ボタンと右ボタンを入れ替え、終了時に戻す。
 * Enter キー押しで終了 */

#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>

unsigned char *g_buf = 0;
int g_num = 0, g_left = -1, g_right = -1;

/* 左ボタンと右ボタンを入れ替え */

static void _set_mapping(Display *disp)
{
	unsigned char tmp;
	int ret;

	if(g_num == -1)
		return;
	else if(g_left == -1 || g_right == -1)
	{
		printf("unfound left or right button\n");
		g_num = -1;
		return;
	}

	tmp = g_buf[g_left];
	g_buf[g_left] = g_buf[g_right];
	g_buf[g_right] = tmp;

	ret = XSetPointerMapping(disp, g_buf, g_num);

	if(ret == MappingSuccess)
		printf("Mapping: success\n");
	else if(ret == MappingBusy)
	{
		printf("Mapping: busy\n");
		g_num = -1;
	}
}

/* 起動時 */

static void _mapping(Display *disp)
{
	int i;

	g_num = XGetPointerMapping(disp, 0, 0);

	g_buf = (unsigned char *)malloc(g_num);

	//現在の値

	XGetPointerMapping(disp, g_buf, g_num);

	printf("-- current --\n");

	for(i = 0; i < g_num; i++)
	{
		printf("[%d] %d\n", i, g_buf[i]);

		if(g_buf[i] == Button1)
			g_left = i;
		else if(g_buf[i] == Button3)
			g_right = i;
	}

	printf("-------------\n");

	//左ボタンと右ボタンを入れ替え

	_set_mapping(disp);
}

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window win;
	XEvent ev;
	KeySym key;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	//ウィンドウ作成

	attr.background_pixel = 0;
	attr.event_mask = ButtonPressMask | ButtonReleaseMask | KeyPressMask;

	win = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, 200, 200, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWEventMask, &attr);

	XMapWindow(disp, win);

	//マッピング

	_mapping(disp);

	//イベント

	while(1)
	{
		XNextEvent(disp, &ev);

		switch(ev.type)
		{
			case ButtonPress:
			case ButtonRelease:
				printf("* [%s] button(%u)\n",
					(ev.type == ButtonPress)? "Press": "Release",
					ev.xbutton.button);

				fflush(stdout);
				break;
			case MappingNotify:
				printf("- [Mapping] request(%d)\n", ev.xmapping.request);
				fflush(stdout);
				break;
			case KeyPress:
				key = XLookupKeysym((XKeyEvent *)&ev, 0);
				if(key == XK_Return || key == XK_KP_Enter) goto END;
				break;
		}
	}

	//
END:
	_set_mapping(disp);
	free(g_buf);

	XCloseDisplay(disp);

	return 0;
}

