/* キーのパッシブグラブ
 *
 * ルートウィンドウに対して F11 キーをグラブ。
 * Shift+F11 を押すと、キー押し時にグラブを解除する。
 * それ以外では、キーが離された時にグラブが解除される。
 * 
 * ウィンドウ内でボタンが押されたら終了 */

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include "util.h"

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window win,root;
	XEvent ev;
	int keycode;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	root = DefaultRootWindow(disp);

	//ウィンドウ作成

	attr.background_pixel = 0;
	attr.event_mask = ButtonPressMask
		| KeyPressMask | KeyReleaseMask | FocusChangeMask;

	win = XCreateWindow(disp, root,
		0, 0, 200, 200, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWEventMask, &attr);

	XMapWindow(disp, win);

	//グラブ

	keycode = XKeysymToKeycode(disp, XK_F11);

	XGrabKey(disp, keycode, AnyModifier, root,
		False, GrabModeAsync, GrabModeAsync);

	printf("root: 0x%lx\nwindow: 0x%lx\ngrab: %d\n", root, win, keycode);

	//イベント

	while(1)
	{
		XNextEvent(disp, &ev);

		switch(ev.type)
		{
			case KeyPress:
			case KeyRelease:
				printf("- [Key%s] window(0x%lx) x(%d) y(%d) keycode(%u)\n",
					(ev.type == KeyPress)? "Press": "Release",
					ev.xkey.window, ev.xkey.x, ev.xkey.y, ev.xkey.keycode);

				put_state(ev.xkey.state);

				if(ev.type == KeyPress && ev.xkey.keycode == keycode
					&& (ev.xkey.state & ShiftMask))
				{
					XUngrabKeyboard(disp, ev.xkey.time);
					printf("# ungrab\n");
				}

				fflush(stdout);
				break;
			case FocusIn:
			case FocusOut:
				printf("* [Focus%s] window(0x%lx) mode(%d) detail(%d)\n",
					(ev.type == FocusIn)? "In": "Out",
					ev.xkey.window, ev.xfocus.mode, ev.xfocus.detail);
				fflush(stdout);
				break;
			case ButtonPress:
				goto END;
		}
	}

END:
	XCloseDisplay(disp);

	return 0;
}

