/* XImage
 *
 * 左ボタン押しで描画内容を変更。
 * それ以外のボタンが押されたら終了 */

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "util.h"

#define WIDTH  200
#define HEIGHT 200

int g_cur_type = 0;

/* バッファにピクセル値をセット */

static void _setpixel(XImage *img,uint8_t *buf,uint32_t pixcol)
{
	//マシンとイメージのバイトオーダーが同じであると想定

	switch(img->bits_per_pixel)
	{
		case 32:
			*((uint32_t *)buf) = pixcol;
			break;
		case 24:
			if(img->byte_order == LSBFirst)
			{
				buf[0] = (uint8_t)pixcol;
				buf[1] = (uint8_t)(pixcol >> 8);
				buf[2] = (uint8_t)(pixcol >> 16);
			}
			else
			{
				buf[0] = (uint8_t)(pixcol >> 16);
				buf[1] = (uint8_t)(pixcol >> 8);
				buf[2] = (uint8_t)pixcol;
			}
			break;
		case 16:
			*((uint16_t *)buf) = pixcol;
			break;
	}
}

/* グラデーションを描画 */

static void _draw_image(XImage *img,int type)
{
	uint8_t *pd,*pdY;
	int ix,iy,bytes,n;
	uint32_t pixcol;

	pdY = (uint8_t *)img->data;
	bytes = img->bits_per_pixel / 8;

	for(iy = 0; iy < img->height; iy++)
	{
		pd = pdY;

		n = iy * 255 / (img->height - 1);
		
		pixcol = rgb_to_pixel_sep(
			(type == 0)? n: 0,
			(type == 1)? n: 0,
			(type == 2)? n: 0);
		
		for(ix = 0; ix < img->width; ix++)
		{
			_setpixel(img, pd, pixcol);
			
			pd += bytes;
		}

		pdY += img->bytes_per_line;
	}
}

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window win;
	XEvent ev;
	XImage *ximg = NULL;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//ウィンドウ作成

	attr.event_mask = ButtonPressMask | ExposureMask;

	win = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, WIDTH, HEIGHT, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWEventMask, &attr);

	//XImage

	ximg = XCreateImage(disp, DefaultVisualOfScreen(g_disp.screen), g_disp.depth,
		ZPixmap, 0, NULL, WIDTH, HEIGHT, BitmapPad(disp), 0);

	if(!ximg)
	{
		printf("failed XCreateImage\n");
		goto END;
	}

	printf("depth: %d\nbits_per_pixel: %d\nbytes_per_line: %d\n",
		ximg->depth, ximg->bits_per_pixel, ximg->bytes_per_line);

	ximg->data = (char *)malloc(ximg->bytes_per_line * HEIGHT);
	if(!ximg->data) goto END;

	_draw_image(ximg, g_cur_type);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		switch(ev.type)
		{
			case Expose:
				XPutImage(disp, win, g_disp.gc, ximg,
					ev.xexpose.x, ev.xexpose.y,
					ev.xexpose.x, ev.xexpose.y,
					ev.xexpose.width, ev.xexpose.height);

				printf("[Expose] (%d,%d)-(%dx%d)\n",
					ev.xexpose.x, ev.xexpose.y, ev.xexpose.width, ev.xexpose.height);
				fflush(stdout);
				break;
			case ButtonPress:
				if(ev.xbutton.button == Button1)
				{
					g_cur_type++;
					if(g_cur_type == 3) g_cur_type = 0;
				
					_draw_image(ximg, g_cur_type);

					XPutImage(disp, win, g_disp.gc, ximg,
						0, 0, 0, 0, WIDTH, HEIGHT);
				}
				else
					goto END;
		}
	}

END:
	if(ximg) XDestroyImage(ximg);

	XCloseDisplay(disp);

	return 0;
}

