/* MIT-SHM
 *
 * ポインタボタンが押されたら終了 */

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include "util.h"

#define WIDTH  200
#define HEIGHT 200

static void _draw_image(XImage *img)
{
	uint8_t *pd,*pdY;
	int ix,iy,bytes;
	uint32_t pixcol;

	pdY = (uint8_t *)img->data;
	bytes = img->bits_per_pixel / 8;

	for(iy = 0; iy < img->height; iy++)
	{
		pd = pdY;

		pixcol = rgb_to_pixel_sep(
			iy * 255 / (img->height - 1), 0, 0);
		
		for(ix = 0; ix < img->width; ix++)
		{
			ximage_setbuf(img, pd, pixcol);
			
			pd += bytes;
		}

		pdY += img->bytes_per_line;
	}
}

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window win;
	XEvent ev;
	XImage *ximg = NULL;
	XShmSegmentInfo si;
	int completion_type;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	set_display(disp);

	//MIT-SHM

	if(!XShmQueryExtension(disp))
	{
		printf("unsupported MIT-SHM\n");
		XCloseDisplay(disp);
		return 1;
	}

	completion_type = XShmGetEventBase(disp) + ShmCompletion;

	//ウィンドウ作成

	attr.event_mask = ButtonPressMask | ExposureMask;

	win = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, WIDTH, HEIGHT, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWEventMask, &attr);

	//----- XImage

	//XImage 作成

	ximg = XShmCreateImage(disp, NULL, g_disp.depth,
		ZPixmap, NULL, &si, WIDTH, HEIGHT);

	if(!ximg)
	{
		printf("failed XCreateImage\n");
		goto END;
	}

	printf("depth: %d\nbits_per_pixel: %d\nbytes_per_line: %d\n",
		ximg->depth, ximg->bits_per_pixel, ximg->bytes_per_line);

	//共有メモリを作り、識別子を取得 (負の値で失敗)

	si.shmid = shmget(IPC_PRIVATE,
		ximg->bytes_per_line * ximg->height, IPC_CREAT | 0777);

	printf("shmid: %d\n", si.shmid);

	//共有メモリをプロセスのアドレス空間にアタッチ

	si.shmaddr = ximg->data = shmat(si.shmid, 0, 0);

	si.readOnly = False;

	//共有メモリセグメントを X サーバーにアタッチ

	XShmAttach(disp, &si);

	printf("shmseg: 0x%lx\n", si.shmseg);

	//

	_draw_image(ximg);

	//---- イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(ev.type == completion_type)
		{
			printf("[ShmCompletion] shmseg:0x%lx\n",
				((XShmCompletionEvent *)&ev)->shmseg);
			fflush(stdout);
			continue;
		}

		switch(ev.type)
		{
			case Expose:
				XShmPutImage(disp, win, g_disp.gc, ximg,
					ev.xexpose.x, ev.xexpose.y,
					ev.xexpose.x, ev.xexpose.y,
					ev.xexpose.width, ev.xexpose.height, True);

				printf("[Expose] (%d,%d)-(%dx%d)\n",
					ev.xexpose.x, ev.xexpose.y, ev.xexpose.width, ev.xexpose.height);
				fflush(stdout);
				break;
			case ButtonPress:
				goto END;
		}
	}

END:
	if(ximg)
	{
		XShmDetach(disp, &si);
		XDestroyImage(ximg);

		//共有メモリをプロセスのアドレス空間から切り離す
		shmdt(si.shmaddr);
		//共有メモリの解放
		shmctl(si.shmid, IPC_RMID, 0);
	}

	XCloseDisplay(disp);

	return 0;
}

