/* 入力メソッド - XIC */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include "util.h"

void _put_text(XKeyEvent *ev,XIC ic)
{
	char m[64],*buf;
	KeySym keysym;
	Status ret;
	int len;

	buf = m;

	len = XmbLookupString(ic, ev, m, 63, &keysym, &ret);

	//バッファ長さが足りない

	if(ret == XBufferOverflow)
	{
		buf = (char *)malloc(len + 1);
		if(!buf) return;

		len = XmbLookupString(ic, ev, buf, len, &keysym, &ret);
	}

	//表示

	buf[len] = 0;

	if(ret == XLookupBoth || ret == XLookupKeySym)
		printf("# KeySym: %lu\n", keysym);

	if(ret == XLookupBoth || ret == XLookupChars)
	{
		if(len == 1 && !(buf[0] >= 0x20 && buf[0] < 0x7f))
			//ASCII 制御文字
			printf("# <%d>\n", buf[0]);
		else
			printf("# '%s'\n", buf);
	}

	if(buf != m) free(buf);
}

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window win;
	XEvent ev;
	XIM im = None;
	XIC ic = None;
	XIMStyle style;
	unsigned long mask;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	init_locale();

	//IM 開く

	im = XOpenIM(disp, NULL, NULL, NULL);
	if(!im)
	{
		printf("failed XOpenIM\n");
		goto END;
	}

	style = get_xim_style_nothing(im);
	if(!style)
	{
		printf("unsupported IM style\n");
		goto END;
	}

	//ウィンドウ作成

	attr.background_pixel = 0;
	attr.event_mask = ButtonPressMask | FocusChangeMask
		| KeyPressMask | KeyReleaseMask;

	win = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, 200, 200, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWEventMask, &attr);

	printf("window: 0x%lx\n", win);

	//IC 作成

	ic = XCreateIC(im,
		XNInputStyle, style,
		XNClientWindow, win,
		XNFocusWindow, win,
		(void *)0);

	if(!ic) goto END;

	//イベントマスク

	mask = 0;
	XGetICValues(ic, XNFilterEvents, &mask, (void *)0);

	printf("event mask: 0x%lx\n", mask);

	XSelectInput(disp, win, mask | attr.event_mask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(XFilterEvent(&ev, None))
		{
			printf("-[filter] window(0x%lx) ", ev.xany.window);

			switch(ev.type)
			{
				case KeyPress:
					printf("<KeyPress> keycode(%d)\n",
						ev.xkey.keycode);
					break;
				case KeyRelease:
					printf("<KeyRelease> keycode(%d)\n",
						ev.xkey.keycode);
					break;
				case ClientMessage:
					printf("<ClientMessage>\n");
					break;
				default:
					printf("<%d>\n", ev.type);
					break;
			}
			continue;
		}

		switch(ev.type)
		{
			case KeyPress:
				printf("[KeyPress] keycode(%d)\n", ev.xkey.keycode);
				_put_text((XKeyEvent *)&ev, ic);
				break;
			case KeyRelease:
				printf("[KeyRelease] keycode(%d)\n", ev.xkey.keycode);
			case FocusIn:
				XSetICFocus(ic);
				printf("[FocusIn] window(0x%lx) mode(%d)\n",
					ev.xfocus.window, ev.xfocus.mode);
				break;
			case FocusOut:
				XUnsetICFocus(ic);
				printf("[FocusOut] window(0x%lx) mode(%d)\n",
					ev.xfocus.window, ev.xfocus.mode);
				break;
			case ButtonPress:
				goto END;
		}
	}

	//
END:
	if(ic) XDestroyIC(ic);
	if(im) XCloseIM(im);

	XCloseDisplay(disp);

	return 0;
}
