/* 入力メソッド - XIMPreeditPosition */

#include <stdio.h>
#include <X11/Xlib.h>
#include "util.h"


/* 前編集開始 */

static int _preedit_start(XIC ic,XPointer client_data,XPointer call_data)
{
	printf("{start}\n");

	//前編集文字列の最大サイズ (-1 で制限なし)
	return -1;
}

/* 前編集終了 */

static void _preedit_done(XIC ic,XPointer client_data,XPointer call_data)
{
	printf("{done}\n");
}

/* 前編集描画 */

static void _preedit_draw(XIC ic,XPointer client_data,XIMPreeditDrawCallbackStruct *p)
{
	XIMFeedback *fb;
	int i;

	printf("--- {draw} ---\n"
		"caret: %d\n" "chg_first: %d\n" "chg_length: %d\n",
		p->caret, p->chg_first, p->chg_length);

	if(!p->text)
		printf("text: NULL\n");
	else
	{
		printf("[text]\n" "length: %d\n" "is_wchar: %d\n",
			p->text->length, p->text->encoding_is_wchar);

		if(p->text->string.multi_byte)
		{
			if(p->text->encoding_is_wchar)
				printf("string: '%ls'\n", p->text->string.wide_char);
			else
				printf("string: '%s'\n", p->text->string.multi_byte);
		}

		if(p->text->feedback)
		{
			fb = p->text->feedback;
			printf("feedback:");

			for(i = 0; i < p->text->length; i++)
				printf(" 0x%lx", *(fb++));

			printf("\n");
		}
	}

	printf("--------\n");
}

/* 前編集カーソル */

static void _preedit_caret(XIC ic,XPointer client_data,XIMPreeditCaretCallbackStruct *p)
{
	printf("{caret}\n" "position: %d\n" "direction: %d\n" "style: %d\n",
		p->position, p->direction, p->style);
}

/* XIC 作成 */

static XIC _create_xic(XIM im,XIMStyle style,Window win)
{
	XIC ic;
	XIMCallback cb[4];
	XVaNestedList valist;

	cb[0].callback = (XIMProc)_preedit_start;
	cb[1].callback = (XIMProc)_preedit_done;
	cb[2].callback = (XIMProc)_preedit_draw;
	cb[3].callback = (XIMProc)_preedit_caret;

	valist = XVaCreateNestedList(0,
		XNPreeditStartCallback, cb,
		XNPreeditDoneCallback, cb + 1,
		XNPreeditDrawCallback, cb + 2,
		XNPreeditCaretCallback, cb + 3,
		(void *)0);

	//XIC 作成

	ic = XCreateIC(im,
		XNInputStyle, style,
		XNClientWindow, win,
		XNFocusWindow, win,
		XNPreeditAttributes, valist,
		(void *)0);

	XFree(valist);

	return ic;
}

int main(int argc,char **argv)
{
	Display *disp;
	XSetWindowAttributes attr;
	Window win;
	XEvent ev;
	XIM im = None;
	XIC ic = None;
	XIMStyle style;
	unsigned long mask;
	
	disp = XOpenDisplay(NULL);
	if(!disp) return 1;

	init_locale();

	//IM 開く

	im = XOpenIM(disp, NULL, NULL, NULL);
	if(!im)
	{
		printf("failed XOpenIM\n");
		goto END;
	}

	style = get_xim_style(im, XIMPreeditCallbacks);
	if(!style)
	{
		printf("unsupported XIMPreeditCallbacks\n");
		goto END;
	}

	//ウィンドウ作成

	attr.background_pixel = 0;
	attr.event_mask = ButtonPressMask | FocusChangeMask
		| KeyPressMask | KeyReleaseMask;

	win = XCreateWindow(disp, DefaultRootWindow(disp),
		0, 0, 200, 200, 0,
		CopyFromParent, CopyFromParent, CopyFromParent,
		CWBackPixel | CWEventMask, &attr);

	printf("window: 0x%lx\n", win);

	//IC 作成

	ic = _create_xic(im, style, win);
	if(!ic) goto END;

	//イベントマスク

	mask = 0;
	XGetICValues(ic, XNFilterEvents, &mask, (void *)0);

	XSelectInput(disp, win, mask | attr.event_mask);

	//イベント

	XMapWindow(disp, win);

	while(1)
	{
		XNextEvent(disp, &ev);

		if(XFilterEvent(&ev, None))
			continue;

		switch(ev.type)
		{
			case KeyPress:
				printf("[KeyPress] keycode(%d)\n", ev.xkey.keycode);
				put_xic_text(&ev, ic);
				break;
			case KeyRelease:
				printf("[KeyRelease] keycode(%d)\n", ev.xkey.keycode);
			case FocusIn:
				XSetICFocus(ic);
				break;
			case FocusOut:
				XUnsetICFocus(ic);
				break;
			case ButtonPress:
				goto END;
		}
	}

	//
END:
	if(ic) XDestroyIC(ic);
	if(im) XCloseIM(im);

	XCloseDisplay(disp);

	return 0;
}
